@@ -27,7 +27,10 @@
 #include "background-manager.h"
 #include "background-manager-glue.h"
 
+#include <gtk/gtkmain.h>
+
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkx.h>
 
 /*SAW (allocation watchdog facilities)*/
 #include <osso-mem.h>
@@ -42,12 +45,12 @@
 #define HILDON_HOME_IMAGE_FORMAT           "png"
 #define HILDON_HOME_IMAGE_ALPHA_FULL       255
 #define HILDON_HOME_GCONF_MMC_COVER_OPEN   "/system/osso/af/mmc-cover-open"
-#define HILDON_HOME_TASKNAV_WIDTH          80
-#define HILDON_HOME_TITLEBAR_HEIGHT        60
 
 #define HILDON_HOME_ENV_MMC_MOUNTPOINT     "MMC_MOUNTPOINT"
 
 
+extern GMainLoop *main_loop;
+
 GQuark
 background_manager_error_quark (void)
 {
@@ -455,22 +458,22 @@ create_background_from_color (const GdkColor  *src,
   return dest;
 }
 
-/* this gets rather convoluted because we want to create a background
- * pixbuf of the same size of the screen and we must account the white
- * space left for the task navigator; so, we must shift the image of
- * HILDON_HOME_TASKNAV_WIDTH pixels on the x axis.
- */
 static GdkPixbuf *
 create_background_from_pixbuf (const GdkPixbuf  *src,
                                const GdkColor   *fill,
                                BackgroundMode    mode,
                                gint              width,
                                gint              height,
+                               gint              top_offset,
+                               gint              bottom_offset,
+                               gint              left_offset,
+                               gint              right_offset,
                                GError          **error)
 {
   GdkPixbuf *dest = NULL;
   gint src_width, src_height;
   gint dest_x, dest_y;
+  gint area_width, area_height;
   gdouble scaling_ratio;
   gdouble off_x, off_y;
 
@@ -489,143 +492,115 @@ create_background_from_pixbuf (const GdkPixbuf  *src,
 
   src_width = gdk_pixbuf_get_width (src);
   src_height = gdk_pixbuf_get_height (src);
+  area_width = width - left_offset - right_offset;
+  area_height = height - top_offset - bottom_offset;
 
   g_debug ("*** background: (w:%d, h:%d), mode: %d",
-	   src_width,
-	   src_height,
-	   mode);
+           src_width,
+           src_height,
+           mode);
 
-  if (src_width == (width - HILDON_HOME_TASKNAV_WIDTH) &&
-      src_height == height)
+  if (src_width == area_width &&
+      src_height == area_height)
     {
       gdk_pixbuf_composite (src,
-		            dest,
-			    0, 0,
-			    width, height,
-			    HILDON_HOME_TASKNAV_WIDTH, 0,
-			    1.0, 1.0,
-			    GDK_INTERP_NEAREST,
-			    0xFF);
+                            dest,
+                            0, 0,
+                            width, height,
+                            left_offset, top_offset,
+                            1.0, 1.0,
+                            GDK_INTERP_NEAREST,
+                            0xFF);
       if (!dest)
         {
           g_set_error (error, BACKGROUND_MANAGER_ERROR,
-		       BACKGROUND_MANAGER_ERROR_SYSTEM_RESOURCES,
-		       "Unable to composite the background color with the image");
+                       BACKGROUND_MANAGER_ERROR_SYSTEM_RESOURCES,
+                       "Unable to composite the background color with the image");
 
-	  return NULL;
-	}
+          return NULL;
+        }
 
       g_debug ("*** We got a background pixbuf");
 
       return dest;
     }
-  
+
   switch (mode)
     {
-    case BACKGROUND_CENTERED:
-      if (src_width < (width - HILDON_HOME_TASKNAV_WIDTH))
-        {
-          dest_x = MAX (HILDON_HOME_TASKNAV_WIDTH,
-                        (width - HILDON_HOME_TASKNAV_WIDTH - src_width)
-                         / 2
-                         + HILDON_HOME_TASKNAV_WIDTH);
-	}
-      else
-	{
-	  dest_x = MAX (HILDON_HOME_TASKNAV_WIDTH,
-                        (width - HILDON_HOME_TASKNAV_WIDTH - src_width)
-                        / 2
-                        + HILDON_HOME_TASKNAV_WIDTH);
-	}
+      case BACKGROUND_CENTERED:
+          off_x = (area_width - src_width) / 2 + left_offset;
+          off_y = (area_height - src_height) / 2 + top_offset;
 
-      if (src_height < height)
-        {
-          dest_y = MAX (0, (height - src_height) / 2);
-	}
-      else
-        {
-	  dest_y = MAX (0, (height - src_height) / 2);
-	}
-      
-      off_x = (width - HILDON_HOME_TASKNAV_WIDTH - src_width)
-              / 2
-              + HILDON_HOME_TASKNAV_WIDTH;
-      off_y = (height-src_height) / 2;
-      
-      gdk_pixbuf_composite (src, dest,
-		            dest_x, dest_y,
-		            MIN (src_width, width - HILDON_HOME_TASKNAV_WIDTH),
-			    MIN (src_height, height),
-			    off_x, off_y,
-			    1.0, 1.0,
-			    GDK_INTERP_NEAREST,
-			    HILDON_HOME_IMAGE_ALPHA_FULL);
-      break;
-    case BACKGROUND_SCALED:
-      scaling_ratio = MIN ((gdouble) ((gdouble) (width - HILDON_HOME_TASKNAV_WIDTH) / src_width),
-		           (gdouble) height / src_height);
-      dest_x = (gint) (MAX (HILDON_HOME_TASKNAV_WIDTH,
-                HILDON_HOME_TASKNAV_WIDTH +
-			    (width
-			     - HILDON_HOME_TASKNAV_WIDTH
-			     - scaling_ratio
-			     * src_width) / 2));
-      dest_y = (gint) (MAX (0,
-			    (height
-			     - scaling_ratio
-			     * src_height) / 2));
+          dest_x = MAX (left_offset, off_x);
 
-      gdk_pixbuf_composite (src, dest,
-			    dest_x, dest_y,
-		        scaling_ratio * src_width,
-			    scaling_ratio * src_height,
-			    MAX (HILDON_HOME_TASKNAV_WIDTH,
-                 HILDON_HOME_TASKNAV_WIDTH +
-				 (width
-				  - HILDON_HOME_TASKNAV_WIDTH
-				  - scaling_ratio
-				  * src_width) / 2),
-			    MAX (0, 
-				 (height
-				  - scaling_ratio
-				  * src_height) / 2),
-			    scaling_ratio, scaling_ratio,
-			    GDK_INTERP_NEAREST,
-			    HILDON_HOME_IMAGE_ALPHA_FULL);
-      break;
-    case BACKGROUND_TILED:
-        for (dest_x = HILDON_HOME_TASKNAV_WIDTH;
-	     dest_x < width;
-	     dest_x += src_width)
-          {
-            for (dest_y = 0;
-	         dest_y < height;
-                 dest_y += src_height)
-              {
-                gdk_pixbuf_composite (src, dest,
-				      dest_x, dest_y,
-				      MIN (src_width, width - dest_x),
-				      MIN (src_height, height - dest_y),
-				      dest_x, dest_y,
-				      1.0, 1.0,
-				      GDK_INTERP_NEAREST,
-				      HILDON_HOME_IMAGE_ALPHA_FULL);
-              }
-	  }
-      break;
-    case BACKGROUND_STRETCHED:
-      gdk_pixbuf_composite (src, dest,
-	      		    HILDON_HOME_TASKNAV_WIDTH, 0,
-			    width - HILDON_HOME_TASKNAV_WIDTH, height,
-			    HILDON_HOME_TASKNAV_WIDTH, 0,
-			    (gdouble) (width-HILDON_HOME_TASKNAV_WIDTH) / src_width,
-			    ((gdouble) height) / src_height,
-			    GDK_INTERP_NEAREST,
-			    HILDON_HOME_IMAGE_ALPHA_FULL);
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
+          dest_y = MAX (top_offset, off_y);
+
+          gdk_pixbuf_composite (src, dest,
+                                dest_x, dest_y,
+                                MIN (src_width, area_width),
+                                MIN (src_height, area_height),
+                                off_x, off_y,
+                                1.0, 1.0,
+                                GDK_INTERP_NEAREST,
+                                HILDON_HOME_IMAGE_ALPHA_FULL);
+          break;
+      case BACKGROUND_SCALED:
+          scaling_ratio = MIN (((gdouble) area_width / src_width),
+                               (gdouble) area_height / src_height);
+          dest_x = (gint) (MAX (left_offset,
+                                left_offset +
+                                (area_width
+                                 - scaling_ratio
+                                 * src_width) / 2));
+          dest_y = (gint) (MAX (top_offset,
+                                top_offset + 
+                                 (area_height
+                                 - scaling_ratio
+                                 * src_height) / 2));
+
+          gdk_pixbuf_composite (src, dest,
+                                dest_x, dest_y,
+                                scaling_ratio * src_width,
+                                scaling_ratio * src_height,
+                                dest_x,
+                                dest_y,
+                                scaling_ratio, scaling_ratio,
+                                GDK_INTERP_NEAREST,
+                                HILDON_HOME_IMAGE_ALPHA_FULL);
+          break;
+      case BACKGROUND_TILED:
+          for (dest_x = left_offset;
+               dest_x < width - right_offset;
+               dest_x += src_width)
+            {
+              for (dest_y = top_offset;
+                   dest_y < height - bottom_offset;
+                   dest_y += src_height)
+                {
+                  gdk_pixbuf_composite (src, dest,
+                                        dest_x, dest_y,
+                                        MIN (src_width, width - dest_x),
+                                        MIN (src_height, height - dest_y),
+                                        dest_x, dest_y,
+                                        1.0, 1.0,
+                                        GDK_INTERP_NEAREST,
+                                        HILDON_HOME_IMAGE_ALPHA_FULL);
+                }
+            }
+          break;
+      case BACKGROUND_STRETCHED:
+          gdk_pixbuf_composite (src, dest,
+                                left_offset, top_offset,
+                                area_width, area_height,
+                                left_offset, top_offset,
+                                (gdouble) area_width  / src_width,
+                                (gdouble) area_height / src_height,
+                                GDK_INTERP_NEAREST,
+                                HILDON_HOME_IMAGE_ALPHA_FULL);
+          break;
+      default:
+          g_assert_not_reached ();
+          break;
     }
 
   return dest;
@@ -647,11 +622,16 @@ composite_background (const GdkPixbuf  *bg_image,
                       gint              window_width,
                       gint              window_height,
                       gboolean          cancellable,
+                      gint              top_offset,
+                      gint              bottom_offset,
+                      gint              left_offset,
+                      gint              right_offset,
                       GError          **error)
 {
   GError *bg_error;
   GdkPixbuf *pixbuf;
   GdkPixbuf *compose;
+  gint titlebar_height = 0;
 
   g_debug ("Compositing background image...");
 
@@ -664,6 +644,10 @@ composite_background (const GdkPixbuf  *bg_image,
                                               mode,
                                               window_width,
                                               window_height,
+                                              top_offset,
+                                              bottom_offset,
+                                              left_offset,
+                                              right_offset,
                                               &bg_error);
     }
   else
@@ -687,6 +671,8 @@ composite_background (const GdkPixbuf  *bg_image,
     {
       compose = load_image_from_file (titlebar_path, cancellable, &bg_error);
 
+      titlebar_height = gdk_pixbuf_get_height (compose);
+
       if (bg_error)
         {
           g_warning ("Unable to load titlebar pixbuf: %s", bg_error->message);
@@ -704,14 +690,20 @@ composite_background (const GdkPixbuf  *bg_image,
       else
         {
           g_debug ("Compositing titlebar");
+          /* Scale it horizontally */
+          double x_scale;
+
+          x_scale = (double)(window_width - left_offset - right_offset) /
+                    gdk_pixbuf_get_width (compose);
 
           gdk_pixbuf_composite (compose,
                                 pixbuf,
-                                HILDON_HOME_TASKNAV_WIDTH, 0,
-                                gdk_pixbuf_get_width (compose),
+                                left_offset, top_offset,
+                                window_width - left_offset - right_offset,
                                 gdk_pixbuf_get_height (compose),
-                                HILDON_HOME_TASKNAV_WIDTH, 0,
-                                1.0, 1.0,
+                                left_offset, top_offset,
+                                x_scale,
+                                1.0,
                                 GDK_INTERP_NEAREST,
                                 HILDON_HOME_IMAGE_ALPHA_FULL);
 
@@ -747,8 +739,7 @@ composite_background (const GdkPixbuf  *bg_image,
           g_debug ("Compositing sidebar (w:%d, h:%d)",
                    width, height);
 
-          sidebar_height = window_height
-              - HILDON_HOME_TITLEBAR_HEIGHT;
+          sidebar_height = window_height - titlebar_height;
           if (height != sidebar_height)
             {
               GdkPixbuf *scaled;
@@ -771,12 +762,12 @@ composite_background (const GdkPixbuf  *bg_image,
 
           gdk_pixbuf_composite (compose,
                                 pixbuf,
-                                HILDON_HOME_TASKNAV_WIDTH,
-                                HILDON_HOME_TITLEBAR_HEIGHT,
+                                left_offset,
+                                top_offset + titlebar_height,
                                 gdk_pixbuf_get_width (compose),
                                 gdk_pixbuf_get_height (compose),
-                                HILDON_HOME_TASKNAV_WIDTH,
-                                0,
+                                left_offset,
+                                top_offset,
                                 1.0, 1.0,
                                 GDK_INTERP_NEAREST,
                                 HILDON_HOME_IMAGE_ALPHA_FULL);
@@ -842,32 +833,55 @@ background_manager_set_background (BackgroundManager   *manager,
                                    guint16              green,
                                    guint16              blue,
                                    BackgroundMode       mode,
+                                   gint32               top_offset,
+                                   gint32               bottom_offset,
+                                   gint32               left_offset,
+                                   gint32               right_offset,
+                                   gint                *pixmap_xid,
                                    GError             **error)
 {
   BackgroundManagerPrivate *priv;
   BackgroundData           *data;
+  GdkDisplay               *display;
   GdkWindow                *window;
   GdkPixbuf                *image = NULL;
   GdkPixbuf                *background = NULL;
   GdkColormap              *colormap;
   GdkPixmap                *pixmap  = NULL;
-  GdkBitmap                *bitmask = NULL;
+/*  GdkBitmap                *bitmask = NULL;*/
   GError                   *local_error = NULL;
   gint                      width, height;
-  FILE *file;
-
-  file = fopen ("/tmp/background-manager.log", "a");
-  fprintf (file, "started applying the background\n");
-  fflush (file);
+  const gchar              *display_name;
+  GdkGC                    *gc;
+  /*XImage                   *ximage;*/
 
   g_debug ("set_background on %s", filename);
+
+  display_name = g_getenv ("DISPLAY");
+  if (!display_name)
+    {
+      g_set_error (error,
+                   background_manager_error_quark (),
+                   BACKGROUND_MANAGER_ERROR_DISPLAY,
+                   "Could not open display");
+      return FALSE;
+    }
+
+  display = gdk_display_open (display_name);
+  if (!display)
+    {
+      g_set_error (error,
+                   background_manager_error_quark (),
+                   BACKGROUND_MANAGER_ERROR_DISPLAY,
+                   "Could not open display %s",
+                   display_name);
+      return FALSE;
+    }
   
-  window = gdk_window_foreign_new (window_xid);
+  window = gdk_window_foreign_new_for_display (display, window_xid);
 
   if (!window)
     {
-  fprintf (file, "No window!\n");
-  fflush (file);
       g_set_error (error,
                    background_manager_error_quark (),
                    BACKGROUND_MANAGER_ERROR_WINDOW,
@@ -877,13 +891,9 @@ background_manager_set_background (BackgroundManager   *manager,
     }
   
   priv = BACKGROUND_MANAGER_GET_PRIVATE (manager);
-  fprintf (file, "Before new data\n");
-  fflush (file);
 
   background_data_free (priv->current);
   
-  fprintf (file, "after free\n");
-  fflush (file);
   priv->current = data = background_data_new ();
 
   data->color->red   = red;
@@ -897,35 +907,21 @@ background_manager_set_background (BackgroundManager   *manager,
     data->top_bar   = g_strdup (top_bar);
   if (left_bar)
     data->left_bar  = g_strdup (left_bar);
-  
-  fprintf (file, "Before load_image_from_uri, %s\n", data->image_uri);
-  fflush (file);
 
   if (data->image_uri && data->image_uri[0])
     {
-      fprintf (file, "Before load_image_from_uri %s\n", data->image_uri);
-      fflush (file);
-
       image = load_image_from_uri (data->image_uri,
                                    TRUE, 
                                    TRUE,
                                    &local_error);
       if (local_error)
         {
-          fprintf (file, "Error: %s\n", local_error->message);
-          fflush (file);
-          fclose (file);
           g_propagate_error (error, local_error);
           return FALSE;
         }
     }
 
-  fprintf (file, "Before get_size\n");
-  fflush (file);
   gdk_drawable_get_size (GDK_DRAWABLE (window), &width, &height);
-  
-  fprintf (file, "Before composite_background\n");
-  fflush (file);
 
   background = composite_background (image,
                                      data->color,
@@ -935,9 +931,11 @@ background_manager_set_background (BackgroundManager   *manager,
                                      width,
                                      height,
                                      TRUE,
+                                     top_offset,
+                                     bottom_offset,
+                                     left_offset,
+                                     right_offset,
                                      &local_error);
-  fprintf (file, "After composite_background\n");
-  fflush (file);
 
   if (image)
     g_object_unref (image);
@@ -950,27 +948,94 @@ background_manager_set_background (BackgroundManager   *manager,
   
   colormap = gdk_drawable_get_colormap (GDK_DRAWABLE (window));
 
+#if 0
   gdk_pixbuf_render_pixmap_and_mask_for_colormap (background,
                                                   colormap,
                                                   &pixmap,
                                                   &bitmask,
                                                   0);
+#endif
+  gc = gdk_gc_new (GDK_DRAWABLE (window));
+  gdk_flush ();
+  pixmap = gdk_pixmap_new (GDK_DRAWABLE (window),
+                           width,
+                           height,
+                           -1);
+                           /*gdk_colormap_get_visual (colormap)->depth);*/
+  gdk_flush ();
+
+#if 0
+
+  g_debug ("Window depth is %i", gdk_drawable_get_depth (GDK_DRAWABLE (window)));
+  g_debug ("bixbuf has %i bits per color", gdk_pixbuf_get_bits_per_sample (background);
+
+  ximage = XCreateImage (GDK_DISPLAY_XDISPLAY(display),
+                         GDK_VISUAL_XVISUAL (gdk_drawable_get_visual (GDK_DRAWABLE (window))),
+                         32,
+/*                       gdk_drawable_get_depth (GDK_DRAWABLE (window))*/
+                         ZPixmap,
+                         0,
+                         gdk_pixbuf_get_pixels (background),
+                         width,
+                         height,
+                         32,
+                         4*width);
+
+  XPutImage (GDK_DISPLAY_XDISPLAY(display),
+             GDK_PIXMAP_XID (pixmap),
+             GDK_GC_XGC (gc),
+             ximage,
+             0,
+             0,
+             0,
+             0,
+             width,
+             height);
+
+  XFree (ximage);
+#else
+
+
+
+  gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap), colormap);
+  gdk_flush ();
+  gdk_draw_pixbuf (pixmap, gc, background, 
+                   0, 0, 0, 0,
+                   gdk_pixbuf_get_width (background),
+                   gdk_pixbuf_get_height (background),
+                   GDK_RGB_DITHER_NORMAL,
+                   0, 0);
+  gdk_flush ();
+  g_object_unref (gc);
+#endif
+
   if (pixmap)
     {
-      gdk_window_set_back_pixmap (window, pixmap, FALSE);
-      g_object_unref (pixmap);
+      if (pixmap_xid)
+        {
+          XSetCloseDownMode (GDK_DISPLAY_XDISPLAY(display), RetainTemporary);
+          *pixmap_xid = (gint)GDK_PIXMAP_XID (pixmap);
+        }
+      else
+        {
+          gdk_window_set_back_pixmap (window, pixmap, FALSE);
+          g_object_unref (pixmap);
+        }
     }
 
+  gdk_display_close (display);
+
+#if 0
   if (bitmask)
     g_object_unref (bitmask);
+#endif
 
   if (background)
     g_object_unref (background);
 
   gdk_window_clear (window);
   gdk_flush ();
-  fprintf (file, "Ended applying the background\n");
-  fflush (file);
-  fclose (file);
+
+  g_idle_add ((GSourceFunc)g_main_loop_quit, main_loop);
   return TRUE;
 }
