@@ -42,6 +42,7 @@ enum
 enum
 {
   SIGNAL_FLIP,
+  SIGNAL_ADD_BUTTON,
   N_SIGNALS
 };
 
@@ -83,6 +84,10 @@ static void hildon_desktop_panel_calc_positions     (HildonDesktopPanel *panel,
 static void hildon_desktop_panel_cadd               (GtkContainer *container,
 					      GtkWidget *widget);
 
+void hildon_desktop_panel_real_add_button (HildonDesktopPanel *panel, 
+					   GtkWidget *widget);
+
+
 static void 
 hildon_desktop_panel_class_init (HildonDesktopPanelClass *panel_class)
 {
@@ -96,7 +101,7 @@ hildon_desktop_panel_class_init (HildonDesktopPanelClass *panel_class)
   panel_class->get_orientation = hildon_desktop_panel_get_orientation;
   panel_class->set_orientation = hildon_desktop_panel_set_orientation;
   panel_class->flip            = hildon_desktop_panel_flip;  
-  panel_class->add_button      = hildon_desktop_panel_add_button;
+  panel_class->add_button      = hildon_desktop_panel_real_add_button;
   panel_class->panel_flipped   = NULL;
 
   container_class->add = hildon_desktop_panel_cadd;
@@ -113,6 +118,16 @@ hildon_desktop_panel_class_init (HildonDesktopPanelClass *panel_class)
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
 
+ signals[SIGNAL_ADD_BUTTON] =
+        g_signal_new("add-button",
+                     G_OBJECT_CLASS_TYPE(object_class),
+                     G_SIGNAL_RUN_FIRST,
+		     G_STRUCT_OFFSET (HildonDesktopPanelClass,add_button),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__POINTER,
+                     G_TYPE_NONE, 0);
+ 
+
   g_object_class_install_property (object_class,
                                    PROP_ORI,
                                    g_param_spec_int ("orientation",
@@ -156,7 +171,7 @@ hildon_desktop_panel_cadd (GtkContainer *container, GtkWidget *widget)
 {
   g_return_if_fail (HILDON_DESKTOP_IS_PANEL (container));
 
-  hildon_desktop_panel_add_button (HILDON_DESKTOP_PANEL (container), widget);
+  hildon_desktop_panel_real_add_button (HILDON_DESKTOP_PANEL (container), widget);
 }
 
 static void 
@@ -751,10 +766,8 @@ hildon_desktop_panel_calc_positions (HildonDesktopPanel *panel,
   gtk_widget_show    (GTK_WIDGET (item)); 
 }
 
-/* Public declarations */
-
 void 
-hildon_desktop_panel_add_button (HildonDesktopPanel *panel, GtkWidget *widget)
+hildon_desktop_panel_real_add_button (HildonDesktopPanel *panel, GtkWidget *widget)
 {
   GtkWidget *panel_widget;
   GtkRequisition req;
@@ -811,6 +824,13 @@ hildon_desktop_panel_add_button (HildonDesktopPanel *panel, GtkWidget *widget)
     gtk_box_pack_start (GTK_BOX (panel), widget, FALSE, FALSE, 0);
   }
 }
+/* Public declarations */
+
+void 
+hildon_desktop_panel_add_button (HildonDesktopPanel *panel, GtkWidget *widget)
+{
+  g_signal_emit_by_name (panel,"add-button",widget);
+}
 
 void 
 hildon_desktop_panel_set_orientation (HildonDesktopPanel *panel, GtkOrientation orientation)
