@@ -1,7 +1,7 @@
 /*
- * This file is part of maemo-af-desktop
+ * This file is part of hildon-desktop
  *
- * Copyright (C) 2005, 2006 Nokia Corporation.
+ * Copyright (C) 2005, 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  * Author: Johan Bilien <johan.bilien@nokia.com>
@@ -27,9 +27,6 @@
 #include "hildon-home-area.h"
 #include "hildon-desktop-marshalers.h"
 
-#include "home-applet-handler.h"
-
-#include <string.h> /* strcmp */
 #include <gtk/gtkfixed.h>
 #include <gtk/gtkicontheme.h>
 #include <hildon-widgets/hildon-defines.h>
@@ -69,10 +66,8 @@ enum
 
 typedef struct HildonHomeAppletPriv_
 {
-  gchar        *desktop_file;
   gboolean      layout_mode;
   HildonHomeAppletResizeType resize_type;
-  HomeAppletHandler *handler;
 
   GdkPixbuf    *close_button;
   GdkWindow    *close_button_window;
@@ -120,9 +115,6 @@ hildon_home_applet_layout_mode_start (HildonHomeApplet *applet);
 static void
 hildon_home_applet_layout_mode_end (HildonHomeApplet *applet);
 
-static void
-hildon_home_applet_desktop_file_changed (HildonHomeApplet *applet);
-
 static gboolean
 hildon_home_applet_button_press_event (GtkWidget *applet,
                                        GdkEventButton   *event);
@@ -262,7 +254,6 @@ hildon_home_applet_class_init (HildonHomeAppletClass * applet_class)
 
   applet_class->layout_mode_start     = hildon_home_applet_layout_mode_start;
   applet_class->layout_mode_end       = hildon_home_applet_layout_mode_end;
-  applet_class->desktop_file_changed  = hildon_home_applet_desktop_file_changed;
 
   parent_class = g_type_class_peek_parent (applet_class);
 
@@ -286,16 +277,6 @@ hildon_home_applet_class_init (HildonHomeAppletClass * applet_class)
                 G_TYPE_NONE,
                 0);
   
-  g_signal_new ("desktop-file-changed",
-                G_OBJECT_CLASS_TYPE (object_class),
-                G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HildonHomeAppletClass, desktop_file_changed),
-                NULL,
-                NULL,
-                g_cclosure_marshal_VOID__VOID,
-                G_TYPE_NONE,
-                0);
-  
   g_signal_new ("background",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_FIRST,
@@ -323,7 +304,7 @@ hildon_home_applet_class_init (HildonHomeAppletClass * applet_class)
                 NULL,
                 NULL,
                 g_cclosure_user_marshal_POINTER__POINTER,
-                GTK_TYPE_WIDGET,
+                G_TYPE_POINTER,
                 1,
                 GTK_TYPE_WIDGET);
 
@@ -331,7 +312,7 @@ hildon_home_applet_class_init (HildonHomeAppletClass * applet_class)
                                "Type of resizability",
                                "Whether the applet can be resized "
                                "vertically, horizontally, or both",
-                               HILDON_HOME_APPLET_RESIZE_TYPE_TYPE,
+                               HILDON_TYPE_HOME_APPLET_RESIZE_TYPE,
                                HILDON_HOME_APPLET_RESIZE_NONE,
                                G_PARAM_READWRITE);
 
@@ -350,16 +331,6 @@ hildon_home_applet_class_init (HildonHomeAppletClass * applet_class)
                                    HILDON_HOME_APPLET_PROPERTY_LAYOUT_MODE,
                                    pspec);
   
-  pspec =  g_param_spec_string ("desktop-file",
-                                "Desktop file name",
-                                "Path the applet's desktop file",
-                                "",
-                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
-
-  g_object_class_install_property (object_class,
-                                   HILDON_HOME_APPLET_PROPERTY_DESKTOP_FILE,
-                                   pspec);
-  
   pspec =  g_param_spec_int ("minimum-width",
                              "Minimum width",
                              "Minimum width for this applet",
@@ -473,9 +444,6 @@ hildon_home_applet_destroy (GtkObject *applet)
       priv->event_window = NULL;
     }
 
-  g_free (priv->desktop_file);
-  priv->desktop_file = NULL;
-  
   if (priv->close_button)
     g_object_unref (priv->close_button);
   priv->close_button = NULL;
@@ -487,13 +455,6 @@ hildon_home_applet_destroy (GtkObject *applet)
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     GTK_OBJECT_CLASS (parent_class)->destroy (applet);
   
-  if (priv->handler)
-    {
-      home_applet_handler_deinitialize (priv->handler);
-      g_object_unref (priv->handler);
-      priv->handler = NULL;
-    }
-
 }
 
 static void
@@ -515,10 +476,6 @@ hildon_home_applet_set_property (GObject      *object,
           hildon_home_applet_set_layout_mode (HILDON_HOME_APPLET (object),
                                               g_value_get_boolean (value));
           break;
-      case HILDON_HOME_APPLET_PROPERTY_DESKTOP_FILE:
-          hildon_home_applet_set_desktop_file (HILDON_HOME_APPLET (object),
-                                               g_value_get_string (value));
-          break;
       case HILDON_HOME_APPLET_PROPERTY_MINIMUM_WIDTH:
           g_object_notify (object, "minimum-width");
           priv->minimum_width = g_value_get_int (value);
@@ -551,9 +508,6 @@ hildon_home_applet_get_property (GObject      *object,
       case HILDON_HOME_APPLET_PROPERTY_LAYOUT_MODE:
           g_value_set_boolean (value, priv->layout_mode);
           break;
-      case HILDON_HOME_APPLET_PROPERTY_DESKTOP_FILE:
-          g_value_set_string (value, priv->desktop_file);
-          break;
       case HILDON_HOME_APPLET_PROPERTY_MINIMUM_WIDTH:
           g_value_set_int (value, priv->minimum_width);
           break;
@@ -638,8 +592,11 @@ static void
 hildon_home_applet_unrealize (GtkWidget *widget)
 {
   HildonHomeAppletPriv      *priv;
+
+  g_return_if_fail (GTK_IS_WIDGET (widget) && GTK_WIDGET_REALIZED (widget));
   
   priv = HILDON_HOME_APPLET_GET_PRIVATE (widget);
+
   
   if (GDK_IS_WINDOW (priv->event_window))
     {
@@ -648,6 +605,10 @@ hildon_home_applet_unrealize (GtkWidget *widget)
       priv->event_window = NULL;
     }
 
+  else
+    g_object_unref (widget->window);
+
+
   if (GTK_WIDGET_CLASS (parent_class)->unrealize)
     GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 
@@ -690,14 +651,14 @@ hildon_home_applet_unmap (GtkWidget *widget)
 
 
 static gboolean
-hildon_home_applet_expose (GtkWidget * w, GdkEventExpose * event)
+hildon_home_applet_expose (GtkWidget *w, GdkEventExpose *event)
 {
   HildonHomeAppletPriv      *priv;
   
   priv = HILDON_HOME_APPLET_GET_PRIVATE (w);
   
   if (GTK_WIDGET_CLASS (parent_class)->expose_event)
-    GTK_WIDGET_CLASS (parent_class)->expose_event(w, event);
+    GTK_WIDGET_CLASS (parent_class)->expose_event (w, event);
   
   if (w->parent && priv->layout_mode)
     {
@@ -761,47 +722,49 @@ hildon_home_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
   HildonHomeAppletPriv      *priv;
   priv = HILDON_HOME_APPLET_GET_PRIVATE (HILDON_HOME_APPLET(widget));
 
+  widget->allocation = *allocation;
+  
   if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_NO_WINDOW (widget))
     {
-      widget->allocation = *allocation;
 
-      if (GTK_WIDGET_REALIZED (widget))
-        {
-          if (GDK_IS_WINDOW (priv->resize_handle_window))
-            gdk_window_move (priv->resize_handle_window,
-                             allocation->width 
-                             - APPLET_RESIZE_HANDLE_WIDTH,
-                             allocation->height 
-                             - APPLET_RESIZE_HANDLE_HEIGHT);
+      if (GDK_IS_WINDOW (priv->resize_handle_window))
+        gdk_window_move (priv->resize_handle_window,
+                         allocation->width 
+                         - APPLET_RESIZE_HANDLE_WIDTH,
+                         allocation->height 
+                         - APPLET_RESIZE_HANDLE_HEIGHT);
 
-          if (GDK_IS_WINDOW (widget->window))
-            gdk_window_move_resize (widget->window,
-                                    allocation->x,
-                                    allocation->y,
-                                    allocation->width,
-                                    allocation->height);
+      if (GDK_IS_WINDOW (widget->window))
+        gdk_window_move_resize (widget->window,
+                                allocation->x,
+                                allocation->y,
+                                allocation->width,
+                                allocation->height);
 
-          if (GDK_IS_WINDOW (priv->event_window))
-            gdk_window_move_resize (priv->event_window,
-                                    0,
-                                    0,
-                                    allocation->width,
-                                    allocation->height);
-        }
+      if (GDK_IS_WINDOW (priv->event_window))
+        gdk_window_move_resize (priv->event_window,
+                                0,
+                                0,
+                                allocation->width,
+                                allocation->height);
 
-      if (GTK_BIN (widget)->child)
-        {
-          GtkAllocation child_allocation = {0};
-          
-          child_allocation.width  = allocation->width;
-          child_allocation.height = allocation->height;
-
-          gtk_widget_size_allocate (GTK_BIN (widget)->child, &child_allocation);
-        }
     }
 
-  else if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
-    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+  if (GTK_BIN (widget)->child)
+    {
+      GtkAllocation child_allocation = {0};
+
+      child_allocation.width  = allocation->width;
+      child_allocation.height = allocation->height;
+
+      if (GTK_WIDGET_NO_WINDOW (widget))
+        {
+          child_allocation.x = allocation->x;
+          child_allocation.y = allocation->y;
+        }
+
+      gtk_widget_size_allocate (GTK_BIN (widget)->child, &child_allocation);
+    }
 }
 
 static void
@@ -1012,76 +975,6 @@ hildon_home_applet_layout_mode_end (HildonHomeApplet *applet)
 
 }
 
-static void
-hildon_home_applet_desktop_file_changed (HildonHomeApplet *applet)
-{
-  HildonHomeAppletPriv *priv;
-  GtkWidget *applet_widget;
-  gboolean horizontal, vertical;
-  gint width, height;
-  gint state_size = 0;
-  void *state_data = NULL;
-  
-  priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
-
-  if (!priv->desktop_file)
-    return;
-
-  /* Replace with Desktop Plugin Manager */
-  
-  priv->handler = home_applet_handler_new (priv->desktop_file,
-                                           state_data,
-                                           &state_size);
-
-  if (!priv->handler)
-    return;
-
-  applet_widget = home_applet_handler_get_widget (priv->handler);
-  
-  if (!applet_widget)
-    return;
-
-  if (GTK_IS_WIDGET (applet_widget))
-    gtk_container_add (GTK_CONTAINER (applet), applet_widget);
-
-  home_applet_handler_get_resizable (priv->handler, &horizontal, &vertical);
-
-  if (horizontal && vertical)
-    hildon_home_applet_set_resize_type (applet, HILDON_HOME_APPLET_RESIZE_BOTH);
-  else if (horizontal)
-    hildon_home_applet_set_resize_type (applet,
-                                        HILDON_HOME_APPLET_RESIZE_HORIZONTAL);
-  else if (vertical)
-    hildon_home_applet_set_resize_type (applet,
-                                        HILDON_HOME_APPLET_RESIZE_VERTICAL);
-  else
-    hildon_home_applet_set_resize_type (applet, HILDON_HOME_APPLET_RESIZE_NONE);
-
-#if 0
-  home_applet_handler_get_size (priv->handler, &width, &height);
-  gtk_widget_set_size_request (GTK_WIDGET (applet), width, height);
-#endif
-  
-  home_applet_handler_get_minimum_size (priv->handler, &width, &height);
-
-  if (width > 0)
-    {
-      g_object_notify (G_OBJECT (applet), "minimum-width");
-      priv->minimum_width = width;
-    }
-
-  if (height > 0)
-    {
-      g_object_notify (G_OBJECT (applet), "minimum-height");
-      priv->minimum_height = height;
-    }
-
-  /* FIXME: This should not be called, but some applets (clock)
-   * currently require this */
-  gtk_widget_show_all (applet_widget);
-
-}
-
 static gboolean
 hildon_home_applet_visibility_notify_event (GtkWidget *applet,
                                             GdkEventVisibility *event)
@@ -1390,22 +1283,6 @@ hildon_home_applet_new (void)
   return GTK_WIDGET(newapplet);
 }
 
-GtkWidget *
-hildon_home_applet_new_with_plugin (const gchar *desktop_file)
-{
-  HildonHomeAppletPriv      *priv;
-  HildonHomeApplet *newapplet = g_object_new (HILDON_TYPE_HOME_APPLET,
-                                              "desktop-file",
-                                              desktop_file,   
-                                              NULL);
-  /* FIXME */
-  priv = HILDON_HOME_APPLET_GET_PRIVATE (newapplet);
-  if (!priv->handler)
-    return NULL;
-    
-  return GTK_WIDGET (newapplet);
-}
-
 gboolean
 hildon_home_applet_get_layout_mode (HildonHomeApplet *applet)
 {
@@ -1467,55 +1344,21 @@ hildon_home_applet_set_resize_type (HildonHomeApplet *applet,
     }
 }
 
-void
-hildon_home_applet_set_desktop_file (HildonHomeApplet *applet,
-                                     const gchar *desktop_file)
-{
-  HildonHomeAppletPriv      *priv;
-  g_return_if_fail (applet);
-
-  priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
-
-  if ((desktop_file && !priv->desktop_file) || 
-      (!desktop_file && priv->desktop_file) ||
-      strcmp (desktop_file, priv->desktop_file))
-    {
-      g_object_notify (G_OBJECT (applet), "desktop-file");
-      g_free (priv->desktop_file);
-      priv->desktop_file = g_strdup (desktop_file);
-        
-      g_signal_emit_by_name (G_OBJECT (applet), "desktop-file-changed");
-    }
-}
-
-const gchar *
-hildon_home_applet_get_desktop_file (HildonHomeApplet *applet)
-{
-  HildonHomeAppletPriv      *priv;
-  g_return_val_if_fail (applet, FALSE);
-  
-  priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
-  
-  return priv->desktop_file;
-}
-
 GtkWidget *
 hildon_home_applet_get_settings_menu_item (HildonHomeApplet *applet)
 {
   HildonHomeAppletPriv      *priv;
   GtkWindow                 *window;
+  GtkWidget                 *item = NULL;
   g_return_val_if_fail (applet, NULL);
   
   priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
   
   window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (applet)));
 
-  return NULL;
-  if (!priv->handler)
-    return NULL;
+  g_signal_emit_by_name (applet, "settings", window, &item);
 
-
-  return home_applet_handler_settings (priv->handler, window);
+  return item;
 }
 
 void
@@ -1554,21 +1397,6 @@ hildon_home_applet_save_position (HildonHomeApplet *applet, GKeyFile *keyfile)
 
 }
 
-gint
-hildon_home_applet_find_by_name (HildonHomeApplet *applet, const gchar *name)
-{
-  HildonHomeAppletPriv      *priv;
-  g_return_val_if_fail (applet && name, -2);
-
-  priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
-
-  if (!priv->desktop_file)
-    return -1;
-
-  return strcmp (priv->desktop_file, name);
-  
-}
-
 gboolean
 hildon_home_applet_get_overlaps (HildonHomeApplet *applet)
 {
@@ -1589,11 +1417,8 @@ hildon_home_applet_set_is_background (HildonHomeApplet *applet,
 
   priv = HILDON_HOME_APPLET_GET_PRIVATE (applet);
 
-  if (!priv->handler)
-    return;
-
   if (is_background)
-    home_applet_handler_background (priv->handler);
+    g_signal_emit_by_name (applet, "background");
   else
-    home_applet_handler_foreground (priv->handler);
+    g_signal_emit_by_name (applet, "foreground");
 }
