@@ -28,6 +28,7 @@
 
 #include <hildon-widgets/hildon-note.h>
 #include <string.h> /* strlen */
+#include <errno.h>
 
 
 enum
@@ -255,6 +256,7 @@ hildon_home_area_class_init (HildonHomeAreaClass *klass)
 static void
 hildon_home_area_init (HildonHomeArea *area)
 {
+  gtk_fixed_set_has_window (GTK_FIXED (area), FALSE);
 }
 
 static void
@@ -394,6 +396,10 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
           g_debug ("No layout configuration");
           gtk_fixed_put (GTK_FIXED (area), applet, 0, 0);
         }
+
+      if (priv->layout_mode)
+        hildon_home_applet_set_layout_mode (HILDON_HOME_APPLET (applet),
+                                            TRUE);
     }
 
 }
@@ -457,13 +463,14 @@ hildon_home_area_get_layout_mode (HildonHomeArea *area)
 
 gint
 hildon_home_area_save_configuration (HildonHomeArea *area,
-                                     const gchar *path)
+                                     const gchar *path,
+                                     GError **error)
 {
   GKeyFile *keyfile;
   FILE     *file;
   gchar    *buffer = NULL;
   guint     buffer_size;
-  GError   *error = NULL;
+  GError   *local_error = NULL;
   gint      ret;
 
   keyfile = g_key_file_new ();
@@ -478,16 +485,20 @@ hildon_home_area_save_configuration (HildonHomeArea *area,
     {
       g_warning ("Could not open %s for saving the applet configuration", path);
       g_key_file_free (keyfile);
+      g_set_error (error,
+                   G_FILE_ERROR,
+                   g_file_error_from_errno (errno),
+                   strerror (errno));
       return -1;
     }
 
-  buffer = g_key_file_to_data (keyfile, &buffer_size, &error);
+  buffer = g_key_file_to_data (keyfile, &buffer_size, &local_error);
 
-  if (error)
+  if (local_error)
     {
       g_warning ("Could not create buffer to save applet configuration: %s",
-                 error->message);
-      g_error_free (error);
+                 local_error->message);
+      g_propagate_error (error, local_error);
       if (file)
         fclose (file);
       return 1;
@@ -520,11 +531,12 @@ hildon_home_area_save_configuration (HildonHomeArea *area,
 
 void
 hildon_home_area_load_configuration (HildonHomeArea *area,
-                                     const gchar *path)
+                                     const gchar *path,
+                                     GError **error)
 {
   HildonHomeAreaPriv      *priv;
   GKeyFile *keyfile;
-  GError *error = NULL;
+  GError *local_error = NULL;
   gchar **groups = NULL;
   gint n_groups;
   GList *applets = NULL;
@@ -538,9 +550,9 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
 
   keyfile = g_key_file_new ();
 
-  g_key_file_load_from_file (keyfile, path, G_KEY_FILE_NONE, &error);
+  g_key_file_load_from_file (keyfile, path, G_KEY_FILE_NONE, &local_error);
   
-  if (error) goto cleanup;
+  if (local_error) goto cleanup;
 
   if (priv->layout)
     g_hash_table_destroy (priv->layout);
@@ -561,23 +573,22 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
       x = g_key_file_get_integer (keyfile,
                                   groups[n_groups-1],
                                   HH_APPLET_KEY_X,
-                                  &error);
-      
+                                  &local_error);
       y = g_key_file_get_integer (keyfile,
                                   groups[n_groups-1],
                                   HH_APPLET_KEY_Y,
-                                  &error);
+                                  &local_error);
       
       width = g_key_file_get_integer (keyfile,
                                       groups[n_groups-1],
                                       HH_APPLET_KEY_WIDTH,
-                                      &error);
+                                      &local_error);
       
       height = g_key_file_get_integer (keyfile,
                                        groups[n_groups-1],
                                        HH_APPLET_KEY_HEIGHT,
-                                       &error);
-      if (error) goto cleanup;
+                                       &local_error);
+      if (local_error) goto cleanup;
 
       else
         {
@@ -598,7 +609,7 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
       list_element = g_list_find_custom (applets,
                                          groups[n_groups-1],
                                          (GCompareFunc)
-                                         hildon_home_applet_find_by_name);
+                                         hildon_desktop_item_find_by_id);
 
       if (list_element)
         {
@@ -653,12 +664,12 @@ cleanup:
   if (groups)
     g_strfreev (groups);
 
-  if (error)
+  if (local_error)
     {
       g_warning ("An error occurred when reading HildonHome applets' "
                  "configuration: %s",
-               error->message);
-      g_error_free (error);
+               local_error->message);
+      g_propagate_error (error, local_error);
     }
 
   g_key_file_free (keyfile);
