@@ -1,8 +1,8 @@
 /* -*- mode:C; c-file-style:"gnu"; -*- */
 /*
- * This file is part of maemo-af-desktop
+ * This file is part of hildon-desktop
  *
- * Copyright (C) 2006 Nokia Corporation.
+ * Copyright (C) 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
@@ -25,7 +25,6 @@
 
 
 #include <glib-object.h>
-#include <glib/gi18n.h>
 
 #include <gtk/gtkalignment.h>
 #include <gtk/gtkmenu.h>
@@ -44,9 +43,7 @@
 #include "hildon-home-titlebar.h"
 #include "hildon-home-area.h"
 #include "hildon-home-window.h"
-#include "hildon-home-l10n.h"
 
-#define HILDON_HOME_TITLEBAR_WIDTH	720
 #define HILDON_HOME_TITLEBAR_HEIGHT	60
 #define HILDON_HOME_TITLEBAR_MENU_LABEL_FONT  "osso-TitleFont"
 #define HILDON_HOME_TITLEBAR_MENU_LABEL_COLOR "TitleTextColor"
@@ -66,10 +63,6 @@
 #define LAYOUT_MODE_BUTTON_PADDING_TOP	 15
 #define LAYOUT_MODE_BUTTON_PADDING_RIGHT 3
 
-/* FIXME */
-#define HCP_LIBRARY_PATH                "/usr/lib/hildon-control-panel"
-#define HCP_PLUGIN_PATH_PERSONALISATION HCP_LIBRARY_PATH"/libpersonalisation.so"
-#define HCP_PLUGIN_PATH_CALIBRATION     HCP_LIBRARY_PATH"/tscalib.so"
 
 #define HH_TITLEBAR_MENU_WIDGET_NAME    "menu_force_with_corners"
 
@@ -127,16 +120,15 @@ enum
 {
   PROP_0,
 
-  PROP_MODE
+  PROP_MODE,
+  PROP_MENU,
+  PROP_MENU_TITLE,
+  PROP_LAYOUT_MENU,
+  PROP_LAYOUT_MENU_TITLE
 };
 
 enum
 {
-  SELECT_APPLETS_ACTIVATE,
-  LAYOUT_MODE_ACTIVATE,
-  APPLET_ACTIVATE,
-  SET_BACKGROUND_ACTIVATE,
-  HELP_ACTIVATE,
   LAYOUT_ACCEPT,
   LAYOUT_CANCEL,
   APPLET_ADDED,
@@ -199,10 +191,9 @@ titlebar_layout_menu_detach (GtkWidget *widget,
 {
   HildonHomeTitlebar *titlebar = HILDON_HOME_TITLEBAR (widget);
 
-  titlebar->priv->layout_menu = NULL;
+  titlebar->priv->menu = NULL;
 }
 
-
 static void
 titlebar_menu_position_func (GtkMenu  *menu,
                              gint     *x,
@@ -226,26 +217,7 @@ titlebar_menu_position_func (GtkMenu  *menu,
 }
 
 static void
-select_applets_activate_cb (GtkMenuItem *menu_item,
-                            gpointer     data)
-{
-  g_signal_emit (data, titlebar_signals[SELECT_APPLETS_ACTIVATE], 0);
-}
-
-static gboolean
-settings_insensitive_press_cb (GtkWidget *widget,
-			       GdkEvent  *event,
-			       gpointer   user_data)
-{
-  hildon_banner_show_information (NULL,
-                                  NULL,
-                                  HH_APPLET_SETTINGS_BANNER);
-  
-  return TRUE;
-}
-
-static void
-layout_accept_activate_cb (GtkWidget *widget,
+layout_accept_clicked_cb (GtkWidget *widget,
 			  gpointer   data)
 {
   g_signal_emit (data, titlebar_signals[LAYOUT_ACCEPT], 0);
@@ -258,278 +230,6 @@ layout_cancel_clicked_cb (GtkWidget *widget,
   g_signal_emit (data, titlebar_signals[LAYOUT_CANCEL], 0);
 }
 
-static void
-layout_mode_activate_cb (GtkWidget *widget,
-			 gpointer   data)
-{
-  g_signal_emit (data, titlebar_signals[LAYOUT_MODE_ACTIVATE], 0);
-}
-
-static gboolean
-layout_mode_insensitive_press_cb (GtkWidget *widget,
-				  GdkEvent  *event,
-				  gpointer   user_data)
-{
-  hildon_banner_show_information (NULL,
-                                  NULL,
-                                  HH_LAYOUT_UNAVAIL_BANNER);
-
-  return TRUE;
-}
-
-static void
-execute_cp_applet (HildonHomeTitlebar *titlebar,
-                   const gchar        *applet_path)
-{
-  g_signal_emit (titlebar,
-                 titlebar_signals[APPLET_ACTIVATE],
-                 0,
-                 applet_path);
-
-}
-
-static void
-personalisation_activate_cb (GtkMenuItem *item,
-                             gpointer     user_data)
-{
-  execute_cp_applet (HILDON_HOME_TITLEBAR (user_data),
-                     HCP_PLUGIN_PATH_PERSONALISATION);
-}
-
-static void
-set_background_activate_cb (GtkMenuItem *item,
-                            gpointer     user_data)
-{
-  g_signal_emit (user_data, titlebar_signals[SET_BACKGROUND_ACTIVATE], 0);
-}
-
-static void
-calibration_activate_cb (GtkMenuItem *item,
-                         gpointer     user_data)
-{
-  execute_cp_applet (HILDON_HOME_TITLEBAR (user_data),
-                     HCP_PLUGIN_PATH_CALIBRATION);
-}
-
-static void
-help_activate_cb (GtkMenuItem *menu_item,
-		  gpointer     user_data)
-{
-  g_debug ("Help activated");
-  
-  g_signal_emit (user_data, titlebar_signals[HELP_ACTIVATE], 0);
-}
-
-static void
-build_layout_mode_titlebar_menu (HildonHomeTitlebar *titlebar)
-{
-  HildonHomeTitlebarPrivate *priv;
-  GtkWidget *menu;
-  GtkWidget *mi;
-
-  priv = titlebar->priv;
-
-  menu = gtk_menu_new ();
-  gtk_widget_set_name (menu, HH_TITLEBAR_MENU_WIDGET_NAME);
-  gtk_menu_attach_to_widget (GTK_MENU (menu),
-                             GTK_WIDGET (titlebar),
-                             titlebar_layout_menu_detach);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_SELECT);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect (mi, "activate",
-                    G_CALLBACK (select_applets_activate_cb),
-                   titlebar);
-  gtk_widget_show (mi);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_ACCEPT);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect (mi, "activate",
-                    G_CALLBACK (layout_accept_activate_cb),
-                    titlebar);
-  gtk_widget_show (mi);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_CANCEL);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect (mi, "activate",
-                    G_CALLBACK (layout_cancel_clicked_cb),
-                    titlebar);
-  gtk_widget_show (mi);
-
-  mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_HELP);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  g_signal_connect (mi, "activate",
-                    G_CALLBACK (help_activate_cb),
-                    titlebar);
-  gtk_widget_show (mi);
-
-  priv->layout_menu = menu;
-
-}
-
-static void
-build_titlebar_menu (HildonHomeTitlebar *titlebar)
-{
-  HildonHomeTitlebarPrivate *priv;
-  GtkWidget *menu;
-  GtkWidget *tools_menu;
-  GtkWidget *settings_menu;
-  GtkWidget *menu_item;
-
-  priv = titlebar->priv;
-
-  /* make sure we get called just when the menu has
-   * not already been set
-   */
-  g_assert (priv->menu == NULL);
-
-  menu = gtk_menu_new ();
-  gtk_widget_set_name (menu, HH_TITLEBAR_MENU_WIDGET_NAME);
-  gtk_menu_attach_to_widget (GTK_MENU (menu),
-                             GTK_WIDGET (titlebar),
-                             titlebar_normal_menu_detach);
-  g_signal_connect (menu, "deactivate",
-                    G_CALLBACK (titlebar_menu_deactivate_cb),
-                    titlebar);
-
-  /* 'Select applets' menu item */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_SELECT);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (select_applets_activate_cb),
-		    titlebar);
-  gtk_widget_show (menu_item);
-  priv->select_applets_item = menu_item;
-
-  /* Applet settings menu item */
-  settings_menu = gtk_menu_new ();
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_APPLET_SETTINGS);
-  g_signal_connect (menu_item, "insensitive-press",
-		    G_CALLBACK (settings_insensitive_press_cb),
-		    titlebar);
-  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), settings_menu);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  gtk_widget_set_sensitive (menu_item, FALSE);
-  gtk_widget_show (menu_item);
-  priv->settings_item = menu_item;
-  priv->settings_menu = settings_menu;
-
-  /* layout mode */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_EDIT_LAYOUT);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (layout_mode_activate_cb),
-		    titlebar);
-  g_signal_connect (menu_item, "insensitive-press",
-		    G_CALLBACK (layout_mode_insensitive_press_cb),
-		    titlebar);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  gtk_widget_set_sensitive (menu_item, FALSE);
-  gtk_widget_show (menu_item);
-  priv->layout_mode_item = menu_item;
-  
-  /* tools sub-menu */
-  tools_menu = gtk_menu_new ();
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_TOOLS);
-  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), tools_menu);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  gtk_widget_show (menu_item);
-
-  /* set background */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_SET_BACKGROUND);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (set_background_activate_cb),
-		    titlebar);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-  
-  /* personalisation */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_PERSONALISATION);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (personalisation_activate_cb),
-		    titlebar);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-
-  /* calibration */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_CALIBRATION);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (calibration_activate_cb),
-		    titlebar);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-
-  /* help */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_HELP);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  g_signal_connect (menu_item, "activate",
-		    G_CALLBACK (help_activate_cb),
-		    titlebar);
-  gtk_widget_show (menu_item);
-
-  priv->tools_menu = tools_menu;
-
-  priv->menu = menu;
-}
-
-static void
-ensure_titlebar_menu_status (HildonHomeTitlebar *titlebar,
-                             HildonHomeArea *area)
-{
-  HildonHomeTitlebarPrivate *priv = titlebar->priv;
-  gboolean settings_item_active = FALSE;
-  GList *items, *l;
-
-  if (!priv->menu)
-    build_titlebar_menu (titlebar);
-
-  /* remove old children from the settings menu */
-  items = gtk_container_get_children (GTK_CONTAINER (priv->settings_menu));
-
-  for (l = items; l != NULL; l = l->next)
-    {
-      GtkWidget *item = GTK_WIDGET (l->data);
-
-      gtk_container_remove (GTK_CONTAINER (priv->settings_menu), item);
-    }
-
-  g_list_free (items);
-  items = NULL;
-
-  items = gtk_container_get_children (GTK_CONTAINER (area));
-  if (!items)
-    {
-      /* if no applets are set we disable the layout_mode item
-       * and the settings item.
-       */
-      gtk_widget_set_sensitive (priv->layout_mode_item, FALSE);
-      gtk_widget_set_sensitive (priv->settings_item, FALSE);
-      return;
-    }
-
-  for (l = items; l != NULL; l = l->next)
-    {
-      HildonHomeApplet *applet;
-      GtkWidget *item;
-  
-      if (!HILDON_IS_HOME_APPLET (l->data))
-        continue;
-      applet = HILDON_HOME_APPLET (l->data);
-
-      item = hildon_home_applet_get_settings_menu_item (applet);
-      if (item && GTK_IS_MENU_ITEM (item))
-        {
-          gtk_menu_append (GTK_MENU (priv->settings_menu), item);
-          gtk_widget_show (item);
-
-          settings_item_active = TRUE;
-        }
-    }
-
-  gtk_widget_set_sensitive (priv->layout_mode_item, TRUE);
-  gtk_widget_set_sensitive (priv->settings_item, settings_item_active);
-  g_list_free (items);
-}
-
 static gboolean
 hildon_home_titlebar_button_press_event (GtkWidget      *widget,
                                          GdkEventButton *event)
@@ -569,20 +269,35 @@ hildon_home_titlebar_button_release_event (GtkWidget       *widget,
 
 static void
 hildon_home_titlebar_set_property (GObject      *gobject,
-				   guint         prop_id,
-				   const GValue *value,
-				   GParamSpec   *pspec)
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
 {
   HildonHomeTitlebar *titlebar = HILDON_HOME_TITLEBAR (gobject);
-        g_debug ("Setting titlebar property");
 
   switch (prop_id)
     {
     case PROP_MODE:
-        g_debug ("Setting titlebar mode");
       hildon_home_titlebar_set_mode (titlebar,
 		                     g_value_get_enum (value));
       break;
+    case PROP_MENU:
+      hildon_home_titlebar_set_menu (titlebar,
+                                     GTK_WIDGET (g_value_get_object (value)));
+      break;
+    case PROP_MENU_TITLE:
+      hildon_home_titlebar_set_menu_title (titlebar,
+                                           g_value_get_string (value));
+      break;
+    case PROP_LAYOUT_MENU:
+      hildon_home_titlebar_set_layout_menu (titlebar,
+                                            GTK_WIDGET (g_value_get_object (value)));
+      break;
+
+    case PROP_LAYOUT_MENU_TITLE:
+      hildon_home_titlebar_set_layout_menu_title (titlebar,
+                                                  g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -591,9 +306,9 @@ hildon_home_titlebar_set_property (GObject      *gobject,
 
 static void
 hildon_home_titlebar_get_property (GObject    *gobject,
-				   guint       prop_id,
-				   GValue     *value,
-				   GParamSpec *pspec)
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
 {
   HildonHomeTitlebar *titlebar = HILDON_HOME_TITLEBAR (gobject);
 
@@ -602,6 +317,18 @@ hildon_home_titlebar_get_property (GObject    *gobject,
     case PROP_MODE:
       g_value_set_enum (value, titlebar->priv->mode);
       break;
+    case PROP_MENU:
+      g_value_set_object (value, titlebar->priv->menu);
+      break;
+    case PROP_MENU_TITLE:
+      g_value_set_string (value, titlebar->priv->normal_text);
+      break;
+    case PROP_LAYOUT_MENU:
+      g_value_set_object (value, titlebar->priv->layout_menu);
+      break;
+    case PROP_LAYOUT_MENU_TITLE:
+      g_value_set_string (value, titlebar->priv->layout_text);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -609,25 +336,12 @@ hildon_home_titlebar_get_property (GObject    *gobject,
 }
 
 static void
-hildon_home_titlebar_applet_added (HildonHomeTitlebar *titlebar,
-                                   HildonHomeArea     *area)
-{
-  ensure_titlebar_menu_status (titlebar, area);
-}
-
-static void
-hildon_home_titlebar_applet_removed (HildonHomeTitlebar *titlebar,
-                                     HildonHomeArea     *area)
-{
-  ensure_titlebar_menu_status (titlebar, area);
-}
-
-    static void
 hildon_home_titlebar_class_init (HildonHomeTitlebarClass *klass)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GObjectClass     *gobject_class = G_OBJECT_CLASS (klass);
+  GtkObjectClass   *object_class = GTK_OBJECT_CLASS (klass);
+  GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
+  GParamSpec       *pspec;
 
   gobject_class->finalize = hildon_home_titlebar_finalize;
   gobject_class->set_property = hildon_home_titlebar_set_property;
@@ -638,59 +352,57 @@ hildon_home_titlebar_class_init (HildonHomeTitlebarClass *klass)
   widget_class->button_press_event = hildon_home_titlebar_button_press_event;
   widget_class->button_release_event = hildon_home_titlebar_button_release_event;
 
-  klass->applet_added = hildon_home_titlebar_applet_added;
-  klass->applet_removed = hildon_home_titlebar_applet_removed;
+  pspec = g_param_spec_enum ("mode",
+                             "Mode",
+                             "Titlebar mode",
+                             HILDON_TYPE_HOME_TITLEBAR_MODE,
+                             HILDON_HOME_TITLEBAR_NORMAL,
+                             G_PARAM_READWRITE);
 
   g_object_class_install_property (gobject_class,
                                    PROP_MODE,
-                                   g_param_spec_enum ("mode",
-                                                      "Mode",
-                                                      "Titlebar mode",
-                                                      HILDON_TYPE_HOME_TITLEBAR_MODE,
-                                                      HILDON_HOME_TITLEBAR_NORMAL,
-                                                      G_PARAM_READWRITE));
+                                   pspec);
+
+  pspec = g_param_spec_object ("menu",
+                               "Menu",
+                               "Menu used in normal mode",
+                               GTK_TYPE_MENU,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_MENU,
+                                   pspec);
+  
+  pspec = g_param_spec_object ("layout-menu",
+                               "Layout mode menu",
+                               "Menu used in layout mode",
+                               GTK_TYPE_MENU,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_LAYOUT_MENU,
+                                   pspec);
+  
+  pspec = g_param_spec_string ("menu-title",
+                               "Menu title",
+                               "Menu title used in normal mode",
+                               "",
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_MENU_TITLE,
+                                   pspec);
+  
+  pspec = g_param_spec_string ("layout-menu-title",
+                               "Layout mode menu title",
+                               "Menu title used in layout mode",
+                               "",
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_LAYOUT_MENU_TITLE,
+                                   pspec);
 
-  titlebar_signals[SELECT_APPLETS_ACTIVATE] =
-      g_signal_new ("select-applets-activate",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, select_applets_activate),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-  titlebar_signals[LAYOUT_MODE_ACTIVATE] =
-      g_signal_new ("layout-mode-activate",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, layout_mode_activate),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-  titlebar_signals[APPLET_ACTIVATE] =
-      g_signal_new ("applet-activate",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, applet_activate),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__STRING,
-                    G_TYPE_NONE,
-                    1, G_TYPE_STRING);
-  titlebar_signals[HELP_ACTIVATE] =
-      g_signal_new ("help-activate",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, help_activate),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-  titlebar_signals[SET_BACKGROUND_ACTIVATE] =
-      g_signal_new ("set-background-activate",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, set_background_activate),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
   titlebar_signals[LAYOUT_ACCEPT] =
       g_signal_new ("layout-accept",
                     G_TYPE_FROM_CLASS (klass),
@@ -708,32 +420,10 @@ hildon_home_titlebar_class_init (HildonHomeTitlebarClass *klass)
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE, 0);
 
-  titlebar_signals[APPLET_ADDED] =
-      g_signal_new ("applet-added",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_FIRST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, applet_added),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__POINTER,
-                    G_TYPE_NONE,
-                    1,
-                    HILDON_TYPE_HOME_AREA);
-
-  titlebar_signals[APPLET_REMOVED] =
-      g_signal_new ("applet-removed",
-                    G_TYPE_FROM_CLASS (klass),
-                    G_SIGNAL_RUN_FIRST,
-                    G_STRUCT_OFFSET (HildonHomeTitlebarClass, applet_removed),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__POINTER,
-                    G_TYPE_NONE,
-                    1,
-                    HILDON_TYPE_HOME_AREA);
-
   g_type_class_add_private (gobject_class, sizeof (HildonHomeTitlebarPrivate));
 }
 
-    static void
+static void
 hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
 {
   HildonHomeTitlebarPrivate *priv;
@@ -747,7 +437,7 @@ hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
   priv->menu_popup_status = FALSE;
 
   gtk_widget_set_size_request (GTK_WIDGET (titlebar),
-                               HILDON_HOME_TITLEBAR_WIDTH,
+                               -1,
                                HILDON_HOME_TITLEBAR_HEIGHT);
 
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (titlebar), FALSE);
@@ -767,8 +457,6 @@ hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
   gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
   gtk_widget_show (align);
 
-  priv->normal_text = g_strdup (HH_MENU_TITLE);
-
   priv->label = gtk_label_new (priv->normal_text);
   gtk_widget_set_composite_name (priv->label, "hildon-home-titlebar-label");
   gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
@@ -805,10 +493,11 @@ hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
                                LAYOUT_MODE_BUTTON_SIZE,
                                LAYOUT_MODE_BUTTON_SIZE);
   g_signal_connect (priv->layout_accept, "clicked",
-                    G_CALLBACK (layout_accept_activate_cb),
+                    G_CALLBACK (layout_accept_clicked_cb),
                     titlebar);
   gtk_box_pack_start (GTK_BOX (hbox), priv->layout_accept, FALSE, FALSE, 0);
 
+
   priv->layout_cancel = gtk_button_new_with_label ("");
   g_object_set (priv->layout_cancel,
                 "image", gtk_image_new_from_icon_name (LAYOUT_MODE_CANCEL_BUTTON,
@@ -822,8 +511,6 @@ hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
                     titlebar);
   gtk_box_pack_start (GTK_BOX (hbox), priv->layout_cancel, FALSE, FALSE, 0);
 
-  priv->layout_text = g_strdup (HH_MENU_LAYOUT_TITLE);
-
   gtk_widget_pop_composite_child ();
 }
 
@@ -888,15 +575,9 @@ hildon_home_titlebar_toggle_menu (HildonHomeTitlebar *titlebar)
   switch (priv->mode)
     {
       case HILDON_HOME_TITLEBAR_NORMAL:
-          if (!priv->menu)
-            build_titlebar_menu (titlebar);
-
           menu = GTK_MENU (priv->menu);
           break;
       case HILDON_HOME_TITLEBAR_LAYOUT:
-          if (!priv->layout_menu)
-            build_layout_mode_titlebar_menu (titlebar);
-
           menu = GTK_MENU (priv->layout_menu);
           break;
       default:
@@ -939,3 +620,115 @@ hildon_home_titlebar_toggle_menu (HildonHomeTitlebar *titlebar)
     }
 
 }
+
+void
+hildon_home_titlebar_set_menu (HildonHomeTitlebar *titlebar,
+                               GtkWidget *menu)
+{
+  HildonHomeTitlebarPrivate *priv;
+
+  g_return_if_fail (HILDON_IS_HOME_TITLEBAR (titlebar));
+
+  if (!GTK_IS_MENU (menu))
+    return;
+
+  priv = HILDON_HOME_TITLEBAR_GET_PRIVATE (titlebar);
+
+  if (priv->menu)
+    gtk_menu_detach (GTK_MENU (priv->menu));
+
+  gtk_menu_attach_to_widget (GTK_MENU (menu), 
+                             GTK_WIDGET (titlebar),
+                             titlebar_normal_menu_detach);
+  priv->menu = menu;
+
+  gtk_widget_set_name (menu, HH_TITLEBAR_MENU_WIDGET_NAME);
+  g_signal_connect (menu, "deactivate",
+                    G_CALLBACK (titlebar_menu_deactivate_cb),
+                    titlebar);
+
+  g_object_notify (G_OBJECT (titlebar), "menu");
+}
+
+void
+hildon_home_titlebar_set_layout_menu (HildonHomeTitlebar *titlebar,
+                                      GtkWidget *menu)
+{
+  HildonHomeTitlebarPrivate *priv;
+
+  g_return_if_fail (HILDON_IS_HOME_TITLEBAR (titlebar));
+  
+  if (!GTK_IS_MENU (menu))
+    return;
+
+  priv = HILDON_HOME_TITLEBAR_GET_PRIVATE (titlebar);
+
+  if (priv->layout_menu)
+    gtk_menu_detach (GTK_MENU (priv->layout_menu));
+
+  gtk_menu_attach_to_widget (GTK_MENU (menu), 
+                             GTK_WIDGET (titlebar),
+                             titlebar_layout_menu_detach);
+  priv->layout_menu = menu;
+
+  gtk_widget_set_name (menu, HH_TITLEBAR_MENU_WIDGET_NAME);
+  g_signal_connect (menu, "deactivate",
+                    G_CALLBACK (titlebar_menu_deactivate_cb),
+                    titlebar);
+
+  g_object_notify (G_OBJECT (titlebar), "layout-menu");
+}
+
+void
+hildon_home_titlebar_set_menu_title (HildonHomeTitlebar *titlebar,
+                                     const gchar *label)
+{
+  HildonHomeTitlebarPrivate *priv;
+
+  g_return_if_fail (HILDON_IS_HOME_TITLEBAR (titlebar));
+
+  if (!label)
+    return;
+
+  priv = HILDON_HOME_TITLEBAR_GET_PRIVATE (titlebar);
+
+  if (!priv->normal_text || !g_str_equal (priv->normal_text, label))
+    {
+      g_free (priv->normal_text);
+
+      priv->normal_text = g_strdup (label);
+      
+      if (priv->mode == HILDON_HOME_TITLEBAR_NORMAL)
+        gtk_label_set_text (GTK_LABEL (priv->label), priv->normal_text);
+      
+      g_object_notify (G_OBJECT (titlebar), "menu-title");
+    }
+
+}
+
+void
+hildon_home_titlebar_set_layout_menu_title (HildonHomeTitlebar *titlebar,
+                                            const gchar *label)
+{
+  HildonHomeTitlebarPrivate *priv;
+
+  g_return_if_fail (HILDON_IS_HOME_TITLEBAR (titlebar));
+
+  if (!label)
+    return;
+
+  priv = HILDON_HOME_TITLEBAR_GET_PRIVATE (titlebar);
+
+  if (!priv->layout_text || !g_str_equal (priv->layout_text, label))
+    {
+      g_free (priv->layout_text);
+
+      priv->layout_text = g_strdup (label);
+
+      if (priv->mode == HILDON_HOME_TITLEBAR_LAYOUT)
+        gtk_label_set_text (GTK_LABEL (priv->label), priv->layout_text);
+      
+      g_object_notify (G_OBJECT (titlebar), "layout-menu-title");
+    }
+
+}
