@@ -32,7 +32,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <libosso.h>
 #include <glib.h>
 
 #include <hildon-widgets/hildon-banner.h>
@@ -285,6 +284,8 @@ hildon_home_window_layout_mode_accept (HildonHomeWindow *window)
     }
 
   g_free (filename);
+
+  g_signal_emit_by_name (window, "save", NULL);
 }
 
 static void
@@ -327,7 +328,7 @@ hildon_home_window_show (GtkWidget *widget)
 {
   gtk_widget_realize (widget);
 
-  gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
+/*  gdk_window_set_back_pixmap (widget->window, NULL, FALSE);*/
   gdk_window_set_type_hint (widget->window,
                             GDK_WINDOW_TYPE_HINT_DESKTOP);
   
