@@ -72,7 +72,7 @@ GType statusbar_item_get_type (void)
         item_type = g_type_register_static ( HILDON_DESKTOP_TYPE_PANEL_ITEM,
                                              "StatusbarItem",
                                              &item_info,
-                                             G_TYPE_FLAG_ABSTRACT );
+                                             0);
     }
     
     return item_type;
