@@ -22,10 +22,7 @@
  *
  */
 
-
 #include <string.h>
-#include <libosso.h>
-#include <log-functions.h>
 
 #include <hildon-widgets/hildon-defines.h>
 #include <hildon-widgets/hildon-banner.h>
@@ -168,7 +165,7 @@ hd_wm_watchable_app_new (const char * file)
   
   if (app_name == NULL || (exec_name == NULL && startup_wmclass == NULL))
     {
-      osso_log(LOG_ERR, "HN: Desktop file has invalid fields");
+      g_warning ("HN: Desktop file has invalid fields");
       g_free(app_name);
       g_free(exec_name);
       g_free(startup_wmclass);
