@@ -29,7 +29,6 @@
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h> /* needed by hildon-navigator-main.h */
-#include <libosso.h>
 #include <log-functions.h>
 #include <hildon-widgets/hildon-defines.h>
 #include <hildon-widgets/hildon-banner.h>
@@ -955,7 +954,7 @@ hd_wm_watched_window_attempt_signal_kill (HDWMWatchedWindow *win,
     {
       HN_DBG("... failed.");
       
-      osso_log(LOG_ERR, "Failed to kill pid %d with signal %d",
+      g_warning ("Failed to kill pid %d with signal %d",
 	       pid_result[0], sig);
 
       XFree(pid_result);
