@@ -41,7 +41,6 @@
 #include <hildon-base-lib/hildon-base-dnotify.h>
 
 #include <libosso.h>
-#include <log-functions.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
@@ -386,7 +385,7 @@ hd_wm_dbus_method_call_handler (DBusConnection *connection,
 
     if (dbus_error_is_set (&error))
     {
-      osso_log(LOG_WARNING, "Error getting message args: %s\n", error.message);
+      g_warning ("Error getting message args: %s\n", error.message);
       return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
 
@@ -457,7 +456,7 @@ hd_wm_dbus_signal_handler (DBusConnection *conn, DBusMessage *msg, void *data)
 
     if (dbus_error_is_set(&err))
     {
-	osso_log(LOG_WARNING, "Error getting message args: %s\n",
+	g_warning ("Error getting message args: %s\n",
 		 err.message);
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1015,7 +1014,7 @@ hd_wm_top_service (const gchar *service_name)
 
   if (service_name == NULL)
   {
-    osso_log(LOG_ERR, "There was no service name!\n");
+    g_warning ("There was no service name!\n");
     return FALSE;
   }
 
