@@ -32,8 +32,13 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_LIBOSSO
+#include <libosso.h>
+#endif
+
 #include <hildon-base-lib/hildon-base-dnotify.h>
 #include <libhildondesktop/hildon-desktop-window.h>
+#include <libhildondesktop/hildon-desktop-panel-window-dialog.h>
 
 #include "hd-desktop.h"
 #include "hd-select-plugins-dialog.h"
@@ -52,10 +57,13 @@ G_DEFINE_TYPE (HDDesktop, hd_desktop, G_TYPE_OBJECT);
 
 struct _HDDesktopPrivate 
 {
-  gchar      *config_path;
-  gchar      *config_file;
-  GHashTable *containers;
-  GObject    *pm;
+  gchar           *config_path;
+  gchar           *config_file;
+  GHashTable      *containers;
+  GObject         *pm;
+#ifdef HAVE_LIBOSSO
+  osso_context_t  *osso_context;
+#endif
 };
 
 static void hd_desktop_load_containers (HDDesktop *desktop);
@@ -263,7 +271,10 @@ hd_desktop_select_plugins (HildonDesktopWindow *window, gpointer user_data)
 
   plugin_dir = hildon_desktop_window_get_plugin_dir (window);
 
-  response = hd_select_plugins_dialog_run (loaded_plugins, 
+  response = hd_select_plugins_dialog_run (loaded_plugins,
+#ifdef HAVE_LIBOSSOHELP
+                                           priv->osso_context,
+#endif 
                                            plugin_dir,
                                            &selected_plugins);
 
@@ -327,6 +338,8 @@ hd_desktop_plugin_dir_changed (gchar *plugin_dir, HildonDesktopWindow *window)
   GError *error = NULL;
   GList *plugin_list = NULL, *remove_list, *iter;
 
+  g_debug ("D-notified");
+
   remove_list = gtk_container_get_children (window->container);
 
   dir = g_dir_open (plugin_dir, 0, &error);
@@ -348,6 +361,8 @@ hd_desktop_plugin_dir_changed (gchar *plugin_dir, HildonDesktopWindow *window)
     
     desktop_path = g_build_filename (plugin_dir, filename, NULL);
 
+/*    g_debug ("found plugin %s", desktop_path);*/
+
     found = g_list_find_custom (remove_list, 
                                 desktop_path,
                                 hd_desktop_find_by_id);
@@ -540,12 +555,14 @@ hd_desktop_load_containers (HDDesktop *desktop)
     if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_HOME))
     {
       container = g_object_new (HD_TYPE_HOME_WINDOW,
-                                /* Add libosso context when we have it */
-                                /*"osso-context", osso_context,*/
+#ifdef HAVE_LIBOSSO
+                                "osso-context", priv->osso_context,
+#endif
                                 NULL);
       hildon_home_window_applets_init (HILDON_HOME_WINDOW (container));
     }
-    else if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_BOX))
+    else if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_BOX) ||
+ 	     !g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_EXPANDABLE))
     {
       HildonDesktopPanelWindowOrientation orientation;
       gchar *orientation_str;
@@ -635,15 +652,29 @@ hd_desktop_load_containers (HDDesktop *desktop)
       else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_RIGHT))
         orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_RIGHT; 
       else 
-        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT; 
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT;
 
-      container = g_object_new (HD_TYPE_PANEL_WINDOW,
-                               "x", x,
-                               "y", y,
-                               "width", width,
-                               "height", height,
-                               "orientation", orientation,
-                                NULL);
+      if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_BOX))
+      {
+        container = g_object_new (HD_TYPE_PANEL_WINDOW,
+                                  "x", x,
+                                  "y", y,
+                                  "width", width,
+                                  "height", height,
+                                  "orientation", orientation,
+                                   NULL);
+      }
+      else
+      if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_EXPANDABLE))
+      {
+        container = g_object_new (HILDON_DESKTOP_TYPE_PANEL_WINDOW_DIALOG,
+                                 "x", x,
+                                 "y", y,
+                                 "width", width,
+                                 "height", height,
+                                 "orientation", orientation,
+                                  NULL);
+      }
 
       g_free (orientation_str);
     }
@@ -755,6 +786,10 @@ hd_desktop_init (HDDesktop *desktop)
     desktop->priv->config_file = g_strdup (env_config_file);
   }
 
+#ifdef HAVE_LIBOSSO
+  desktop->priv->osso_context = osso_initialize (PACKAGE, VERSION, TRUE, NULL);
+#endif
+  
   hd_desktop_dnotify_init ();
 
   hd_desktop_watch_file (desktop->priv->config_file, 
