@@ -1,4 +1,3 @@
-/* -*- mode:C; c-file-style:"gnu"; -*- */
 /*
  * This file is part of hildon-desktop
  *
@@ -23,21 +22,31 @@
  *
  */
 
-#include <libosso.h>
-#include <osso-helplib.h>
-#include <hildon-widgets/hildon-file-chooser-dialog.h>
-#include <hildon-widgets/hildon-color-button.h>
-#include <hildon-widgets/hildon-caption.h>
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#ifdef HAVE_LIBOSSOHELP
+#include <osso-helplib.h>
+#endif
+
+#ifdef HAVE_LIBOSSO
+#include <libosso.h>
+#endif
+
+#ifdef HAVE_HILDON_FM
+#include <hildon-widgets/hildon-file-chooser-dialog.h>
+#endif
+
+#include <hildon-widgets/hildon-color-button.h>
+#include <hildon-widgets/hildon-caption.h>
 
 #include <string.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtkliststore.h>
@@ -45,14 +54,13 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcelllayout.h>
 
-#include <libosso.h>
-#include <osso-helplib.h>
 #include <hildon-widgets/hildon-file-chooser-dialog.h>
 #include <hildon-widgets/hildon-color-button.h>
 #include <hildon-widgets/hildon-caption.h>
 
 #include "hd-home-background-dialog.h"
 #include "hd-home-l10n.h"
+#include "hd-home-background.h"
 #include "background-manager/hildon-background-manager.h"
 
 #define BG_DESKTOP_GROUP           "Desktop Entry"
@@ -73,9 +81,6 @@
 #define HILDON_HOME_FILE_CHOOSER_EMPTY_PROP   "empty-text"
 #define HILDON_HOME_FILE_CHOOSER_EMPTY        _("home_li_no_images")
 
-#define HILDON_HOME_SET_BG_RESPONSE_IMAGE    GTK_RESPONSE_YES
-#define HILDON_HOME_SET_BG_RESPONSE_PREVIEW  GTK_RESPONSE_APPLY
-
 #define HILDON_HOME_BACKGROUND_NO_IMAGE  "no_image"
 
 
@@ -94,17 +99,24 @@ enum
 enum
 {
   PROP_0,
-  PROP_BACKGROUND_DIR
+  PROP_BACKGROUND_DIR,
+  PROP_OSSO_CONTEXT,
+  PROP_BACKGROUND
 };
 
 struct _HDHomeBackgroundDialogPrivate
 {
   gchar            *background_dir;
   GtkListStore     *combobox_contents;
-  GtkTreeIter       active_iter;
+  GtkTreePath      *custom_image;
   GtkWidget        *color_button;
   GtkWidget        *img_combo;
   GtkWidget        *mode_combo;
+#ifdef HAVE_LIBOSSOHELP
+  osso_context_t   *osso_context;
+#endif
+  
+  HDHomeBackground *background;
 };
 
 static void
@@ -131,6 +143,38 @@ static void
 hd_home_background_dialog_background_dir_changed 
                                     (HDHomeBackgroundDialog *dialog);
 
+static void
+hd_home_background_dialog_set_background (HDHomeBackgroundDialog *dialog,
+                                          HDHomeBackground *background);
+static void
+hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog);
+
+static void
+hd_home_background_dialog_filename_changed (HDHomeBackgroundDialog *dialog);
+
+static void
+hd_home_background_dialog_color_changed (HDHomeBackgroundDialog *dialog);
+
+static void
+hd_home_background_dialog_mode_changed (HDHomeBackgroundDialog *dialog);
+
+static void
+hd_home_background_dialog_finalize (GObject *object);
+
+static gchar *
+imagename_from_filename (const gchar *filename)
+{
+  gchar *tmp;
+  gchar *imagename;
+
+  tmp = g_filename_from_uri (filename, NULL, NULL);
+  if (!tmp)
+    tmp = g_strdup (filename);
+
+  imagename = g_filename_display_basename (tmp);
+
+  return imagename;
+}
 
 static void
 hd_home_background_dialog_class_init (HDHomeBackgroundDialogClass *klass)
@@ -144,6 +188,7 @@ hd_home_background_dialog_class_init (HDHomeBackgroundDialogClass *klass)
 
   object_class->set_property = hd_home_background_dialog_set_property;
   object_class->get_property = hd_home_background_dialog_get_property;
+  object_class->finalize = hd_home_background_dialog_finalize;
 
   dialog_class->response = hd_home_background_dialog_response;
 
@@ -169,9 +214,28 @@ hd_home_background_dialog_class_init (HDHomeBackgroundDialogClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_BACKGROUND_DIR,
                                    pspec);
+  
+  pspec = g_param_spec_pointer ("osso-context",
+                                "OSSO Context",
+                                "OSSO Context",
+                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (object_class,
+                                   PROP_OSSO_CONTEXT,
+                                   pspec);
+  
+  pspec = g_param_spec_object ("background",
+                               "Background",
+                               "Current background",
+                               HD_TYPE_HOME_BACKGROUND,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  g_object_class_install_property (object_class,
+                                   PROP_BACKGROUND,
+                                   pspec);
+
 
   g_type_class_add_private (klass, sizeof (HDHomeBackgroundDialogPrivate));
-
 }
 
 static void
@@ -208,36 +272,24 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
 
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_window_set_title (GTK_WINDOW (dialog), HH_SET_BG_TITLE);
-
-#if 0
-  /* Add help button */
-  ossohelp_dialog_help_enable (GTK_DIALOG (dialog),
-                               HH_HELP_SET_BACKGROUND,
-                               home_get_osso_context ());
-#endif
-
+  
   /* Hildon Caption HBoxes */
   hbox_color = gtk_hbox_new (FALSE, 10);
   hbox_image = gtk_hbox_new (FALSE, 10);
   hbox_mode  = gtk_hbox_new (FALSE, 10);
-     
-#if 0
-  /* Widgets for Hildon Captions in Set Background Image dialog */
-  background_manager_get_color (bm, &current_color);
-  priv->color_button = hildon_color_button_new_with_color (&current_color);
-#endif
   
   priv->color_button = hildon_color_button_new ();
   
-  priv->img_combo =
-    gtk_combo_box_new_with_model (GTK_TREE_MODEL (priv->combobox_contents));
-  
-#if 0
-  if(!background_manager_get_image_uri (bm) ||
-     g_str_equal(background_manager_get_image_uri (bm),
-		 HILDON_HOME_BACKGROUND_NO_IMAGE))
-#endif
+  g_signal_connect_swapped (priv->color_button, "notify::color",
+                            G_CALLBACK (hd_home_background_dialog_color_changed),
+                            dialog);
   
+  priv->img_combo = gtk_combo_box_new ();
+
+  g_signal_connect_swapped (priv->img_combo, "changed",
+                            G_CALLBACK (hd_home_background_dialog_filename_changed),
+                            dialog);
+
   renderer = gtk_cell_renderer_text_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->img_combo),
                               renderer, 
@@ -258,11 +310,9 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
   gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo), 
                              image_modes[BACKGROUND_TILED]);
   
-#if 0
-  /* Centered is a default scaling value */
-  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_mode_select), 
-			    background_manager_get_mode (bm));
-#endif
+  g_signal_connect_swapped (priv->mode_combo, "changed",
+                            G_CALLBACK (hd_home_background_dialog_mode_changed),
+                            dialog);
 
   /* Hildon captions in Set Background Image dialog */
   group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
@@ -297,6 +347,8 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox_image);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox_mode);
 
+  gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
+
   hildon_caption_set_child_expand (HILDON_CAPTION (image_caption), TRUE);
   hildon_caption_set_child_expand (HILDON_CAPTION (mode_caption), TRUE);
   
@@ -310,6 +362,10 @@ hd_home_background_dialog_set_property (GObject      *gobject,
                                         const GValue *value,
                                         GParamSpec   *pspec)
 {
+  HDHomeBackgroundDialogPrivate    *priv;
+
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (gobject);
+
   switch (prop_id)
   {
     case PROP_BACKGROUND_DIR:
@@ -317,6 +373,24 @@ hd_home_background_dialog_set_property (GObject      *gobject,
                           HD_HOME_BACKGROUND_DIALOG (gobject),
                           g_value_get_string (value));
         break;
+
+    case PROP_OSSO_CONTEXT:
+        priv->osso_context = g_value_get_pointer (value);
+
+#ifdef HAVE_LIBOSSOHELP
+        /* Add help button */
+        if (priv->osso_context)
+          ossohelp_dialog_help_enable (GTK_DIALOG (gobject),
+                                       HH_HELP_SET_BACKGROUND,
+                                       priv->osso_context);
+#endif
+        break;
+        
+    case PROP_BACKGROUND:
+        hd_home_background_dialog_set_background (
+                           HD_HOME_BACKGROUND_DIALOG (gobject),
+                           g_value_get_object (value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
         break;
@@ -338,29 +412,60 @@ hd_home_background_dialog_get_property (GObject      *gobject,
     case PROP_BACKGROUND_DIR:
         g_value_set_string (value, priv->background_dir);
         break;
+
+#ifdef HAVE_LIBOSSOHELP
+    case PROP_OSSO_CONTEXT:
+        g_value_set_pointer (value, priv->osso_context);
+        break;
+#endif
+
+    case PROP_BACKGROUND:
+        g_value_set_object (value, priv->background);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
         break;
   }
 }
 
-#if 0
-static gboolean
-home_bgd_select_file_dialog (GtkWidget   * parent,
-			     GtkComboBox * box,
-			     gboolean      built_in)
+static void
+hd_home_background_dialog_finalize (GObject *object)
 {
-  GtkTreeModel *tree = gtk_combo_box_get_model (box);
-  GtkWidget *dialog;
-  GtkFileFilter *mime_type_filter;
-  gint response;
-  gchar *name_str, *dot;
-  gchar *chooser_name;
-  gchar *image_dir;
-  
-  dialog =
-    hildon_file_chooser_dialog_new_with_properties(
-					  GTK_WINDOW(parent),
+  HDHomeBackgroundDialogPrivate    *priv;
+
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (object);
+
+  if (priv->background)
+    {
+      g_object_unref (priv->background);
+      priv->background = NULL;
+    }
+
+  if (priv->custom_image)
+    {
+      gtk_tree_path_free (priv->custom_image);
+      priv->custom_image = NULL;
+    }
+
+  g_free (priv->background_dir);
+  priv->background_dir = NULL;
+
+}
+
+static void
+hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate    *priv;
+  GtkWidget                        *fdialog;
+  GtkFileFilter                    *mime_type_filter;
+  gint                              response;
+  gchar                            *image_dir;
+
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+#ifdef HAVE_HILDON_FM  
+  fdialog = hildon_file_chooser_dialog_new_with_properties (
+					  GTK_WINDOW (dialog),
 					  HILDON_HOME_FILE_CHOOSER_ACTION_PROP,
 					  GTK_FILE_CHOOSER_ACTION_OPEN,
 					  HILDON_HOME_FILE_CHOOSER_TITLE_PROP,
@@ -370,12 +475,20 @@ home_bgd_select_file_dialog (GtkWidget   * parent,
 					  HILDON_HOME_FILE_CHOOSER_EMPTY_PROP,
 					  HILDON_HOME_FILE_CHOOSER_EMPTY,
 					  NULL);
-
-#if 0
-  /* Add help button */
-  ossohelp_dialog_help_enable(GTK_DIALOG(dialog), 
-			      HILDON_HOME_SELECT_IMAGE_HELP_TOPIC,
-			      home_get_osso_context ());
+#else
+  fdialog = gtk_file_chooser_dialog_new (HILDON_HOME_FILE_CHOOSER_TITLE,
+					 GTK_WINDOW (dialog),
+					 GTK_FILE_CHOOSER_ACTION_OPEN,
+                                         HILDON_HOME_FILE_CHOOSER_SELECT, GTK_RESPONSE_OK, 
+                                         HILDON_HOME_FILE_CHOOSER_EMPTY, GTK_RESPONSE_CANCEL, 
+					 NULL);
+#endif
+  
+#ifdef HAVE_LIBOSSOHELP
+  if (priv->osso_context)
+    ossohelp_dialog_help_enable (GTK_DIALOG(fdialog), 
+                                 HH_HELP_SELECT_IMAGE,
+                                 priv->osso_context);
 #endif
         
   mime_type_filter = gtk_file_filter_new();
@@ -385,126 +498,150 @@ home_bgd_select_file_dialog (GtkWidget   * parent,
   gtk_file_filter_add_mime_type (mime_type_filter, "image/bmp");
   gtk_file_filter_add_mime_type (mime_type_filter, "image/tiff");
   gtk_file_filter_add_mime_type (mime_type_filter, "sketch/png");
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fdialog),
+                               mime_type_filter);
 
-  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog),
-			       mime_type_filter);
-
-  image_dir = g_build_path("/",
-                           g_getenv ("HOME"),
+  image_dir = g_build_path(G_DIR_SEPARATOR_S,
+                           g_get_home_dir (),
                            HILDON_HOME_HC_USER_IMAGE_DIR,
                            NULL);
 
-  if(!gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog),
-                                          image_dir))
+  if (!gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fdialog),
+                                            image_dir))
     g_warning ("Couldn't set default image dir for dialog %s", 
                image_dir);
 
   g_free (image_dir);
   
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  response = gtk_dialog_run (GTK_DIALOG (fdialog));
     
   if (response == GTK_RESPONSE_OK) 
     {
+      const gchar  *filename;
 
-      chooser_name = 
-          gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+      filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fdialog));
 
-      name_str = g_filename_from_uri (chooser_name, NULL, NULL);
-      if (!name_str)
-        name_str = g_strdup (chooser_name);
-
-      /* remove extension */
-      if ((dot = g_strrstr (name_str, ".")) && dot != name_str)
-        *dot = '\0';
-
-      if (name_str && *name_str) 
+      if (filename) 
         {
-          GtkTreeIter iter;            
-          GtkTreePath *path;
-          gint priority = 0;
-          gtk_list_store_append (GTK_LIST_STORE (tree), &iter);
+          GtkTreeModel *model;
+          GtkTreeIter   iter;            
+          gchar        *name;
 
-          gtk_list_store_set (GTK_LIST_STORE (tree),
-                             &iter,
-                             BG_IMAGE_NAME, name_str,
-                             BG_IMAGE_FILENAME, chooser_name,
-                             BG_IMAGE_PRIORITY, G_MAXINT, -1);
+          name = imagename_from_filename (filename);
 
-          gtk_combo_box_set_active_iter (box, &iter);
+          model = GTK_TREE_MODEL (priv->combobox_contents);
 
-          path = gtk_tree_model_get_path (tree, &iter);
-          gtk_tree_path_next (path);
-
-          if (gtk_tree_model_get_iter (tree, &iter, path)) 
+          if (priv->custom_image)
             {
-              priority = home_bgd_get_priority (tree, &iter);
-              if (priority == G_MAXINT) 
-                {
-                  gtk_list_store_remove(GTK_LIST_STORE(tree), &iter);
-
-                }
+              gtk_tree_model_get_iter (model,
+                                       &iter,
+                                       priv->custom_image);
+              gtk_list_store_remove (priv->combobox_contents,
+                                     &iter);
+              gtk_tree_path_free (priv->custom_image);
             }
-          gtk_tree_path_free(path);
+
+          gtk_list_store_append (priv->combobox_contents,
+                                 &iter);
+          
+          priv->custom_image = gtk_tree_model_get_path (model,
+                                                        &iter);
+
+          gtk_list_store_set (priv->combobox_contents,
+                              &iter,
+                              BG_IMAGE_NAME, name?name:filename,
+                              BG_IMAGE_FILENAME, filename,
+                              BG_IMAGE_PRIORITY, G_MAXINT,
+                              -1);
+
+          gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->img_combo),
+                                         &iter);
+          g_free (name);
         }
 
-      /* Clean up */
-      g_free(name_str);
-      gtk_widget_destroy(dialog);    
-
-      return TRUE;
     }  
 
-  gtk_widget_destroy(dialog);
+  gtk_widget_destroy (fdialog);
 
-  return FALSE;
 }
-#endif
 
 static void
 hd_home_background_dialog_response (GtkDialog *dialog, 
                                     gint       arg)
 {
-  HDHomeBackgroundDialogPrivate *priv;
+  HDHomeBackgroundDialog        *background_dialog;
+
+  background_dialog = HD_HOME_BACKGROUND_DIALOG (dialog);
+
+  if (arg == HILDON_HOME_SET_BG_RESPONSE_IMAGE)
+    hd_home_background_dialog_file_select (background_dialog);
+}
+
+static void
+hd_home_background_dialog_filename_changed (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate    *priv;
+  GtkTreeIter                       iter;
+  gchar                            *filename;
+  
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->img_combo),
+                                 &iter);
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->combobox_contents),
+                      &iter,
+                      BG_IMAGE_FILENAME, &filename,
+                      -1);
+
+  if (filename)
+    g_object_set (G_OBJECT (priv->background),
+                  "filename", filename,
+                  NULL);
+  else
+    g_object_set (G_OBJECT (priv->background),
+                  "filename", HD_HOME_BACKGROUND_NO_IMAGE,
+                  NULL);
+
+  g_free (filename);
+}
+
+static void
+hd_home_background_dialog_color_changed (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate    *priv;
+  GdkColor                         *color;
 
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
-#if 0
-  g_debug ("response_data = {\n"
-           "  img_combo    .= %s\n"
-           "  mode_combo   .= %s\n"
-           "  color_button .= %s\n"
-           "  applied      .= %s\n"
-           "  built_in     .= %s\n"
-           "  has_preview  .= %s\n"
-           "}",
-           g_type_name (G_OBJECT_TYPE (d->img_combo)),
-           g_type_name (G_OBJECT_TYPE (d->mode_combo)),
-           g_type_name (G_OBJECT_TYPE (d->color_button)),
-           d->applied ? "<true>" : "<false>",
-           d->built_in ? "<true>" : "<false>",
-           d->has_preview ? "<true>" : "<false>");
-#endif
+  if (!priv->background)
+    return;
 
-  switch (arg) 
-    {
-      case HILDON_HOME_SET_BG_RESPONSE_IMAGE:
-      /*    home_bgd_select_file_dialog (dialog, img_combo, d->built_in);*/
-          break;
-      case HILDON_HOME_SET_BG_RESPONSE_PREVIEW:
-          break;
-      case GTK_RESPONSE_OK:
-          gtk_widget_hide (GTK_WIDGET(dialog));
-          break;
-      case GTK_RESPONSE_CANCEL:
-      case GTK_RESPONSE_DELETE_EVENT:
-          gtk_widget_hide (GTK_WIDGET (dialog));
-          break;
-      default:
-          break;
-    }
+  color =
+   hildon_color_button_get_color (HILDON_COLOR_BUTTON (priv->color_button));
 
+  g_object_set (G_OBJECT (priv->background),
+                "color", color,
+                NULL);
 }
 
+static void
+hd_home_background_dialog_mode_changed (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate    *priv;
+  BackgroundMode                    mode;
+
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+  if (!priv->background)
+    return;
+
+  mode = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->mode_combo));
+
+  g_object_set (G_OBJECT (priv->background),
+                "mode", mode,
+                NULL);
+}
 
 static void
 hd_home_background_dialog_background_dir_changed 
@@ -517,17 +654,21 @@ hd_home_background_dialog_background_dir_changed
   const gchar                   *image_desc_file = NULL;
   GKeyFile                      *kfile;
 
-  g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog) && dir);
+  g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog));
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
   if (priv->combobox_contents)
     gtk_list_store_clear (priv->combobox_contents);
   else
-    priv->combobox_contents =
-      gtk_list_store_new (3,
-                          G_TYPE_STRING, /*localised descriptive name */
-                          G_TYPE_STRING, /* image file path & name */
-                          G_TYPE_INT     /* image priority */);
+    {
+      priv->combobox_contents =
+          gtk_list_store_new (3,
+                              G_TYPE_STRING, /*localised descriptive name */
+                              G_TYPE_STRING, /* image file path & name */
+                              G_TYPE_INT     /* image priority */);
+      gtk_combo_box_set_model (GTK_COMBO_BOX (priv->img_combo),
+                               GTK_TREE_MODEL (priv->combobox_contents));
+    }
 
   if (!priv->background_dir)
     return;
@@ -554,6 +695,10 @@ hd_home_background_dialog_background_dir_changed
                       BG_IMAGE_FILENAME, NULL,
                       BG_IMAGE_PRIORITY, 0, -1);
 
+  /* It's the default */
+  gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->img_combo),
+                                 &iterator);
+
   kfile = g_key_file_new ();
 
   while ((image_desc_file = g_dir_read_name (dir)))
@@ -600,41 +745,6 @@ hd_home_background_dialog_background_dir_changed
 
       if (error) goto end_of_loop;
 
-#if 0
-      current_uri = background_manager_get_image_uri (bm);
-
-      if (current_uri)
-        {
-          gchar *current_path;
-          GError *error = NULL;
-
-          current_path = g_filename_from_uri (current_uri, NULL, &error);
-          if (error)
-            {
-              g_warning ("Unable to get the filename from uri `%s': %s",
-                         current_uri,
-                         error->message);
-
-              g_error_free (error);
-
-              /* we let the background manager deal with it */
-              current_path = g_strdup (current_uri);
-            }
-
-          g_debug ("current filename [%s], processing [%s]",
-                   current_path, image_path);
-
-          if (image_path &&
-              g_str_equal(image_path, current_path)) 
-            {
-              g_debug ("current BG uses built in image");
-              resp_data->built_in = TRUE;
-            }
-
-          g_free (current_path);
-        }
-#endif
-
       gtk_list_store_append (priv->combobox_contents, &iterator);
 
       gtk_list_store_set (priv->combobox_contents,
@@ -668,6 +778,86 @@ end_of_loop:
                                         GTK_SORT_ASCENDING);
 }
 
+static void
+hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate    *priv;
+  gchar                            *filename;
+  GdkColor                         *color;
+  BackgroundMode                    mode;
+
+  g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog));
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+  if (!priv->background)
+    return;
+  
+  g_object_get (G_OBJECT (priv->background),
+                "color", &color,
+                "filename", &filename,
+                "mode", &mode,
+                NULL);
+
+  if (filename && *filename && 
+      !g_str_equal (filename, HD_HOME_BACKGROUND_NO_IMAGE))
+    {
+      gboolean      valid;
+      GtkTreeIter   iter;
+      GtkTreeModel *model;
+
+      model = GTK_TREE_MODEL (priv->combobox_contents);
+
+      valid = gtk_tree_model_get_iter_first (model, &iter);
+
+      while (valid)
+        {
+          gchar *existing_filename = NULL;
+
+          gtk_tree_model_get (model,
+                              &iter,
+                              BG_IMAGE_FILENAME, &existing_filename,
+                              -1);
+
+          if (existing_filename && g_str_equal (existing_filename, filename))
+            break;
+
+          valid = gtk_tree_model_iter_next (model, &iter);
+        }
+
+      if (!valid)
+        {
+          gchar *image_name = imagename_from_filename (filename);
+
+          gtk_list_store_append (priv->combobox_contents, &iter);
+
+          gtk_list_store_set (priv->combobox_contents,
+                              &iter,
+                              BG_IMAGE_FILENAME, filename,
+                              BG_IMAGE_NAME, image_name,
+                              BG_IMAGE_PRIORITY, G_MAXINT,
+                              -1);
+
+          if (priv->custom_image)
+            gtk_tree_path_free (priv->custom_image);
+
+          priv->custom_image = gtk_tree_model_get_path (model, &iter);
+
+          g_free (image_name);
+        }
+
+      gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->img_combo),
+                                     &iter);
+
+      g_free (filename);
+    }
+
+  if (color)
+    hildon_color_button_set_color (HILDON_COLOR_BUTTON (priv->color_button),
+                                   color);
+
+  gtk_combo_box_set_active (GTK_COMBO_BOX (priv->mode_combo), mode);
+}
+
 static void
 hd_home_background_dialog_set_background_dir (HDHomeBackgroundDialog *dialog,
                                               const gchar *dir)
@@ -688,3 +878,34 @@ hd_home_background_dialog_set_background_dir (HDHomeBackgroundDialog *dialog,
   }
 }
 
+static void
+hd_home_background_dialog_set_background (HDHomeBackgroundDialog *dialog,
+                                          HDHomeBackground *background)
+{
+  HDHomeBackgroundDialogPrivate *priv;
+  
+  g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog) &&
+                    HD_IS_HOME_BACKGROUND (background));
+
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+  if (priv->background)
+    g_object_unref (priv->background);
+
+  priv->background = g_object_ref (background);
+
+  g_object_notify (G_OBJECT (dialog), "background");
+  hd_home_background_dialog_sync_from_background (dialog);
+
+}
+
+HDHomeBackground *
+hd_home_background_dialog_get_background (HDHomeBackgroundDialog *dialog)
+{
+  HDHomeBackgroundDialogPrivate *priv;
+
+  g_return_val_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog), NULL);
+  priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
+
+  return priv->background;
+}
