@@ -35,10 +35,12 @@
 
 #include <gdk/gdkcolor.h>
 #include <gdk/gdkwindow.h>
+#include <gdk/gdkpixmap.h>
 #include <gdk/gdkx.h>
 
 #include <glib/gkeyfile.h>
 
+/* Key used in the home-background.conf */
 #define HD_HOME_BACKGROUND_KEY_GROUP            "Hildon Home"
 #define HD_HOME_BACKGROUND_KEY_URI              "BackgroundImage"
 #define HD_HOME_BACKGROUND_KEY_SIDEBAR          "SidebarImage"
@@ -50,7 +52,8 @@
 #define HD_HOME_BACKGROUND_KEY_BLUE             "Blue"
 #define HD_HOME_BACKGROUND_KEY_MODE             "Mode"
 
-#define HD_HOME_BACKGROUND_VALUE_NO_IMAGE       "no-image"
+/* Some pre-defined values */
+#define HD_HOME_BACKGROUND_VALUE_NO_IMAGE       HD_HOME_BACKGROUND_NO_IMAGE
 #define HD_HOME_BACKGROUND_VALUE_CENTERED       "Centered"
 #define HD_HOME_BACKGROUND_VALUE_STRETCHED      "Stretched"
 #define HD_HOME_BACKGROUND_VALUE_SCALED         "Scaled"
@@ -102,8 +105,6 @@ hd_home_background_init (HDHomeBackground *background)
       G_TYPE_INSTANCE_GET_PRIVATE ((background),
                                    HD_TYPE_HOME_BACKGROUND,
                                    HDHomeBackgroundPrivate);
-
-  background->priv->color = g_new0 (GdkColor, 1);
 }
 
 static void
@@ -207,7 +208,8 @@ hd_home_background_finalize (GObject *object)
   g_free (priv->south_border);
   priv->south_border = NULL;
 
-  g_free (priv->color);
+  if (priv->color)
+    gdk_color_free (priv->color);
   priv->color = NULL;
 
 }
@@ -224,7 +226,11 @@ hd_home_background_set_property (GObject       *object,
     {
       case PROP_FILENAME:
           g_free (priv->filename);
-          priv->filename = g_strdup (g_value_get_string (value));
+          if (g_str_equal (g_value_get_string (value),
+                           HD_HOME_BACKGROUND_NO_IMAGE))
+            priv->filename = g_strdup ("");
+          else
+            priv->filename = g_strdup (g_value_get_string (value));
           break;
       case PROP_NORTH_BORDER:
           g_free (priv->north_border);
@@ -246,8 +252,9 @@ hd_home_background_set_property (GObject       *object,
           priv->mode = g_value_get_int (value);
           break;
       case PROP_COLOR:
-          g_free (priv->color);
-          priv->color = g_value_get_pointer (value);
+          if (priv->color)
+            gdk_color_free (priv->color);
+          priv->color = gdk_color_copy (g_value_get_pointer (value));
           break;
       default:
           G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -472,6 +479,10 @@ hd_home_background_load (HDHomeBackground *background,
                                       HD_HOME_BACKGROUND_KEY_GROUP,
                                       HD_HOME_BACKGROUND_KEY_RED,
                                       &local_error);
+
+  if (!priv->color)
+    priv->color = g_new0 (GdkColor, 1);
+
   if (local_error)
     {
       priv->color->red = 0;
@@ -533,15 +544,26 @@ cleanup:
     g_propagate_error (error, local_error);
 }
 
+static void
+free_pixmap (Pixmap pixmap_xid)
+{
+  g_debug ("Freeing pixmap %i\n", (int)pixmap_xid);
+  XFreePixmap (GDK_DISPLAY (), pixmap_xid);
+}
+
 void
 hd_home_background_apply (HDHomeBackground *background,
                           GdkWindow        *window,
+                          GdkRectangle     *area,
                           GError          **error)
 {
   HDHomeBackgroundPrivate  *priv;
   DBusGProxy           *background_manager_proxy;
   DBusGConnection      *connection;
   GError               *local_error = NULL;
+  gint                  pixmap_xid;
+  GdkPixmap            *pixmap = NULL;
+  gint32                top_offset, bottom_offset, right_offset, left_offset;
 
   g_return_if_fail (HD_IS_HOME_BACKGROUND (background) && window);
   priv = background->priv;
@@ -559,18 +581,49 @@ hd_home_background_apply (HDHomeBackground *background,
                                  HILDON_BACKGROUND_MANAGER_OBJECT_PATH,
                                  HILDON_BACKGROUND_MANAGER_INTERFACE);
 
+  top_offset = bottom_offset = right_offset = left_offset = 0;
+  if (area)
+    {
+      gint width, height;
+      gdk_drawable_get_size (GDK_DRAWABLE (window), &width, &height);
+
+      top_offset = area->y;
+      bottom_offset = MAX (0, height - area->height - area->y);
+      left_offset = area->x;
+      right_offset = MAX (0, width- area->width - area->x);
+    }
+
   g_debug ("Trying to apply background to window: %x", (int)GDK_WINDOW_XID(window));
 
+#define S(string) (string?string:"")
   org_maemo_hildon_background_manager_set_background (background_manager_proxy,
                                                       GDK_WINDOW_XID (window),
-                                                      priv->filename,
-                                                      priv->west_border,
-                                                      priv->north_border,
+                                                      S(priv->filename),
+                                                      S(priv->west_border),
+                                                      S(priv->north_border),
                                                       priv->color->red,
                                                       priv->color->green,
                                                       priv->color->blue,
                                                       priv->mode,
+                                                      top_offset,
+                                                      bottom_offset,
+                                                      left_offset,
+                                                      right_offset,
+                                                      &pixmap_xid,
                                                       error);
+#undef S
+
+  if (pixmap_xid)
+    pixmap = gdk_pixmap_foreign_new (pixmap_xid);
+
+  if (pixmap)
+    {
+      gdk_window_set_back_pixmap (window, pixmap, FALSE);
+      g_object_unref (pixmap);
+    }
+  else
+    g_warning ("No such pixmap: %i", pixmap_xid);
+
 }
 
 struct cb_data
@@ -578,24 +631,56 @@ struct cb_data
   HDHomeBackground                 *background;
   HDHomeBackgroundApplyCallback     callback;
   gpointer                          user_data;
+  GdkWindow                        *window;
 };
 
+
 static void
 hd_home_background_apply_async_dbus_callback (DBusGProxy       *proxy,
+                                              gint              pixmap_xid,
                                               GError           *error,
                                               struct cb_data   *data)
 {
-
+  GdkPixmap *pixmap;
+  
   g_debug ("DBUS Callback");
-  if (data->callback)
-    data->callback (data->background, error, data->user_data);
 
+  if (error)
+    {
+      g_warning ("Error when setting background: %s", error->message);
+      goto cleanup;
+    }
+
+  if (!pixmap_xid)
+    {
+      g_warning ("No pixmap id returned");
+      goto cleanup;
+    }
+    
+  pixmap = gdk_pixmap_foreign_new (pixmap_xid);
+
+  if (pixmap)
+    {
+      g_object_weak_ref (G_OBJECT (pixmap),
+                         (GWeakNotify)free_pixmap,
+                         GINT_TO_POINTER((GDK_PIXMAP_XID (pixmap))));
+      gdk_window_set_back_pixmap (data->window, pixmap, FALSE);
+      g_object_unref (pixmap);
+    }
+  else
+    g_warning ("No such pixmap: %i", pixmap_xid);
+
+  if (data->callback)
+    data->callback (data->background, pixmap_xid, error, data->user_data);
+
+cleanup:
   g_free (data);
 }
 
 void
 hd_home_background_apply_async (HDHomeBackground               *background,
                                 GdkWindow                      *window,
+                                GdkRectangle                   *area,
                                 HDHomeBackgroundApplyCallback   cb,
                                 gpointer                        user_data)
 {
@@ -604,6 +689,7 @@ hd_home_background_apply_async (HDHomeBackground               *background,
   DBusGConnection          *connection;
   GError                   *local_error = NULL;
   struct cb_data           *data;
+  gint32                    top_offset, bottom_offset, right_offset, left_offset;
 
   g_return_if_fail (HD_IS_HOME_BACKGROUND (background) && window);
   priv = background->priv;
@@ -612,7 +698,7 @@ hd_home_background_apply_async (HDHomeBackground               *background,
   if (local_error)
     {
       if (cb)
-        cb (background, local_error, user_data);
+        cb (background, 0, local_error, user_data);
       g_error_free (local_error);
       return;
     }
@@ -628,18 +714,64 @@ hd_home_background_apply_async (HDHomeBackground               *background,
   data->callback = cb;
   data->background = background;
   data->user_data = user_data;
+  data->window = window;
+
+  top_offset = bottom_offset = right_offset = left_offset = 0;
+  if (area)
+    {
+      gint width, height;
+      gdk_drawable_get_size (GDK_DRAWABLE (window), &width, &height);
+
+      top_offset = area->y;
+      bottom_offset = MAX (0, height - area->height - area->y);
+      left_offset = area->x;
+      right_offset = MAX (0, width- area->width - area->x);
+    }
+
+  g_debug ("Applying background %s aynchronously",
+           priv->filename);
 
   /* Here goes */
+#define S(string) (string?string:"")
   org_maemo_hildon_background_manager_set_background_async 
                                                 (background_manager_proxy,
                                                  GDK_WINDOW_XID (window),
-                                                 priv->filename,
-                                                 priv->west_border,
-                                                 priv->north_border,
+                                                 S(priv->filename),
+                                                 S(priv->west_border),
+                                                 S(priv->north_border),
                                                  priv->color->red,
                                                  priv->color->green,
                                                  priv->color->blue,
                                                  priv->mode,
+                                                 top_offset,
+                                                 bottom_offset,
+                                                 left_offset,
+                                                 right_offset,
                                                  (org_maemo_hildon_background_manager_set_background_reply) hd_home_background_apply_async_dbus_callback,
                                                  data);
+#undef S
+}
+
+HDHomeBackground *
+hd_home_background_copy (const HDHomeBackground *src)
+{
+  HDHomeBackgroundPrivate  *priv;
+  HDHomeBackground         *dest;
+
+  g_return_val_if_fail (HD_IS_HOME_BACKGROUND (src), NULL);
+
+  priv = src->priv;
+
+  dest = g_object_new (HD_TYPE_HOME_BACKGROUND,
+                       "mode", priv->mode,
+                       "color", priv->color,
+                       "filename", priv->filename,
+                       "west-border", priv->west_border,
+                       "east-border", priv->east_border,
+                       "south-border", priv->south_border,
+                       "north-border", priv->north_border,
+                       NULL);
+
+  return dest;
+
 }
