@@ -25,16 +25,23 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#include "hd-home-l10n.h"
-#include "hd-home-background.h"
 
-#include <libosso.h>
+#ifdef HAVE_LIBOSSOHELP
 #include <osso-helplib.h>
+#endif
+
+#ifdef HAVE_LIBOSSO
+#include <libosso.h>
+#endif
+
 #include <libhildondesktop/hildon-home-area.h>
 #include <libhildondesktop/hildon-home-titlebar.h>
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
 
+#include "hd-home-l10n.h"
+#include "hd-home-background.h"
+#include "hd-home-background-dialog.h"
 
 /* FIXME */
 #define HCP_LIBRARY_PATH                "/usr/lib/hildon-control-panel"
@@ -60,7 +67,9 @@ G_DEFINE_TYPE (HDHomeWindow, hd_home_window, HILDON_TYPE_HOME_WINDOW);
 
 struct _HDHomeWindowPrivate
 {
+#ifdef HAVE_LIBOSSO
   osso_context_t   *osso_context;
+#endif
 
   GtkWidget        *settings_item;
   GtkWidget        *settings_menu;
@@ -100,21 +109,15 @@ hd_home_window_set_property (GObject      *gobject,
                              const GValue *value,
                              GParamSpec   *pspec);
 
+static void
+hd_home_window_notify (GObject     *object,
+                       GParamSpec  *pspec);
+
+
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_set_osso_context (HDHomeWindow *window, osso_context_t *osso);
 
-static void
-hd_home_window_calibration_activate (HDHomeWindow *window);
-
-static void
-hd_home_window_set_background_activate (HDHomeWindow *window);
-
-static void
-hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window);
-
-static void
-hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window);
-
 static void
 hd_home_window_personalisation_activate (HDHomeWindow *window);
 
@@ -123,6 +126,16 @@ hd_home_window_calibration_activate (HDHomeWindow *window);
 
 static void
 hd_home_window_help_activate (HDHomeWindow *window);
+#endif
+
+static void
+hd_home_window_set_background_activate (HDHomeWindow *window);
+
+static void
+hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window);
+
+static void
+hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window);
 
 static GtkWidget *
 hd_home_window_build_main_menu (HDHomeWindow *window);
@@ -172,6 +185,7 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   object_class->constructor  = hd_home_window_constructor;
   object_class->set_property = hd_home_window_set_property;
   object_class->get_property = hd_home_window_get_property;
+  object_class->notify       = hd_home_window_notify;
 
   widget_class->map_event = hd_home_window_map_event;
   widget_class->style_set = hd_home_window_style_set;
@@ -263,7 +277,7 @@ hd_home_window_constructor (GType                   gtype,
   background = g_object_new (HD_TYPE_HOME_BACKGROUND, NULL);
 
   conffile = g_build_path (G_DIR_SEPARATOR_S,
-                           g_getenv ("HOME"),
+                           g_get_home_dir (),
                            HD_DESKTOP_USER_PATH,
                            HD_HOME_BACKGROUND_CONF_FILE,
                            NULL);
@@ -314,12 +328,14 @@ hd_home_window_set_property (GObject      *gobject,
 
   switch (prop_id)
   {
+#ifdef HAVE_LIBOSSO
     case PROP_OSSO_CONTEXT:
       hd_home_window_set_osso_context (window,
                                        (osso_context_t *)
                                        g_value_get_pointer (value));
 
       break;
+#endif
 
     case PROP_BACKGROUND:
       hd_home_window_set_background (window,
@@ -342,10 +358,12 @@ hd_home_window_get_property (GObject    *gobject,
 
   switch (prop_id)
   {
+#ifdef HAVE_LIBOSSO
     case PROP_OSSO_CONTEXT:
       g_value_set_pointer (value, priv->osso_context);
       break;
-
+#endif
+      
     case PROP_BACKGROUND:
       g_value_set_object (value, priv->background);
       break;
@@ -356,11 +374,28 @@ hd_home_window_get_property (GObject    *gobject,
   }
 }
 
+static void
+hd_home_window_notify (GObject     *object,
+                       GParamSpec  *pspec)
+{
+  HDHomeWindowPrivate  *priv;
+  priv = HD_HOME_WINDOW_GET_PRIVATE (object);
+
+  if (g_str_equal (pspec->name, "work-area"))
+    {
+      g_debug ("Work area changed, reapplying the background");
+      hd_home_window_set_background (HD_HOME_WINDOW (object),
+                                     priv->background);
+    }
+}
+
 static void
 background_apply_callback (HDHomeBackground *background,
+                           gint              pixmap_xid,
                            GError           *error,
                            HDHomeWindow     *window)
 {
+  HDHomeWindowPrivate  *priv;
   g_debug ("Background applied!");
 
   if (error)
@@ -372,6 +407,56 @@ background_apply_callback (HDHomeBackground *background,
 
   gdk_window_clear (GTK_WIDGET (window)->window);
   gtk_widget_queue_draw (GTK_WIDGET (window));
+  
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  if (background != priv->background)
+    {
+      if (priv->background)
+        g_object_unref (priv->background);
+      priv->background = g_object_ref (background);
+    }
+ 
+}
+
+static void
+background_apply_and_save_callback (HDHomeBackground *background,
+                                    gint              pixmap_xid,
+                                    GError           *error,
+                                    HDHomeWindow     *window)
+{
+  HDHomeWindowPrivate  *priv;
+  gchar                *conffile;
+  GError               *save_error = NULL;
+
+  background_apply_callback (background, pixmap_xid, error, window);
+
+  /* Do not save if an error occurred */
+  if (error)
+    return;
+
+  conffile = g_build_path (G_DIR_SEPARATOR_S,
+                           g_get_home_dir (),
+                           HD_DESKTOP_USER_PATH,
+                           HD_HOME_BACKGROUND_CONF_FILE,
+                           NULL);
+  
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  
+  hd_home_background_save (priv->background,
+                           conffile,
+                           &save_error);
+
+  if (save_error)
+    {
+      g_warning ("Error when saving background settings to %s: %s",
+                 conffile,
+                 error->message);
+      g_error_free (save_error);
+    }
+
+  g_free (conffile);
+
 }
 
 static gboolean
@@ -379,10 +464,14 @@ hd_home_window_map_event (GtkWidget    *widget,
                           GdkEventAny  *event)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (widget);
+  GdkRectangle         *workarea;
+
+  g_object_get (widget, "work-area", &workarea, NULL);
 
   if (priv->background)
     hd_home_background_apply_async (priv->background,
                                     widget->window,
+                                    workarea,
                                     (HDHomeBackgroundApplyCallback)
                                     background_apply_callback,
                                     widget);
@@ -468,8 +557,12 @@ hd_home_window_style_set (GtkWidget *widget, GtkStyle *old_style)
 
           if (GTK_WIDGET_MAPPED (widget))
             {
+              GdkRectangle *workarea;
+              g_object_get (widget, "work-area", &workarea, NULL);
+
               hd_home_background_apply_async (priv->background,
                                               widget->window,
+                                              workarea,
                                               (HDHomeBackgroundApplyCallback)
                                               background_apply_callback,
                                               widget);
@@ -540,7 +633,8 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
                             window);
   gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
   gtk_widget_show (menu_item);
-  
+
+#ifdef HAVE_LIBOSSO
   /* personalisation */
   menu_item = gtk_menu_item_new_with_label (HH_MENU_PERSONALISATION);
   g_signal_connect_swapped (menu_item, "activate",
@@ -564,7 +658,8 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
                             G_CALLBACK (hd_home_window_help_activate),
                             window);
   gtk_widget_show (menu_item);
-
+#endif
+  
   return menu;
 
 }
@@ -598,16 +693,19 @@ hd_home_window_build_layout_menu (HDHomeWindow *window)
                             window);
   gtk_widget_show (mi);
 
+#ifdef HAVE_LIBOSSO
   mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_HELP);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate",
                             G_CALLBACK (hd_home_window_help_activate),
                             window);
   gtk_widget_show (mi);
-
+#endif
+  
   return menu;
 }
 
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_applet_activate (HDHomeWindow *window,
                                 const gchar *applet_path)
@@ -646,7 +744,9 @@ hd_home_window_applet_activate (HDHomeWindow *window,
       break;
   }
 }
+#endif
 
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_help_activate (HDHomeWindow *window)
 {
@@ -694,28 +794,26 @@ hd_home_window_help_activate (HDHomeWindow *window)
       break;
   }
 }
+#endif
 
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_calibration_activate (HDHomeWindow *window)
 {
-  HDHomeWindowPrivate  *priv;
   hd_home_window_applet_activate (window,
                                   HCP_PLUGIN_PATH_CALIBRATION);
       
-  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  hd_home_background_apply_async (priv->background,
-                                  GTK_WIDGET (window)->window,
-                                  (HDHomeBackgroundApplyCallback)
-                                  background_apply_callback,
-                                  window);
 }
+#endif
 
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_personalisation_activate (HDHomeWindow *window)
 {
   hd_home_window_applet_activate (window,
                                   HCP_PLUGIN_PATH_PERSONALISATION);
 }
+#endif
 
 static void
 hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window)
@@ -792,9 +890,7 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
   g_list_free (items);
 }
 
-
-
-
+#ifdef HAVE_LIBOSSO
 static void
 hd_home_window_set_osso_context (HDHomeWindow *window,
                                  osso_context_t *osso_context)
@@ -826,6 +922,7 @@ hd_home_window_set_osso_context (HDHomeWindow *window,
     }
   }
 }
+#endif
 
 static gboolean
 hd_home_window_destroy_banner (HDHomeWindow *window)
@@ -935,6 +1032,9 @@ hd_home_window_set_background (HDHomeWindow *window,
                                HDHomeBackground *background)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GdkRectangle *workarea;
+
+  g_object_get (window, "work-area", &workarea, NULL);
 
   if (priv->background)
     g_object_unref (priv->background);
@@ -948,6 +1048,7 @@ hd_home_window_set_background (HDHomeWindow *window,
     {
       hd_home_background_apply_async (background,
                                       GTK_WIDGET (window)->window,
+                                      workarea,
                                       (HDHomeBackgroundApplyCallback)
                                       background_apply_callback,
                                       window);
@@ -955,7 +1056,64 @@ hd_home_window_set_background (HDHomeWindow *window,
 
 }
 
+static void
+hd_home_window_set_background_reponse (HDHomeWindow *window,
+                                       gint response,
+                                       GtkDialog *dialog)
+{
+  HDHomeBackground *background;
+  GdkRectangle *workarea;
+
+  g_object_get (window, "work-area", &workarea, NULL);
+
+  background =
+      hd_home_background_dialog_get_background (HD_HOME_BACKGROUND_DIALOG (dialog));
+
+  switch (response)
+    {
+      case GTK_RESPONSE_OK:
+          gtk_widget_hide (GTK_WIDGET(dialog));
+          hd_home_background_apply_async 
+              (background,
+               GTK_WIDGET (window)->window,
+               workarea,
+               (HDHomeBackgroundApplyCallback)background_apply_and_save_callback,
+               window);
+          break;
+      case HILDON_HOME_SET_BG_RESPONSE_PREVIEW:
+          hd_home_background_apply_async 
+              (background,
+               GTK_WIDGET (window)->window,
+               workarea,
+               (HDHomeBackgroundApplyCallback)background_apply_callback,
+               window);
+
+          break;
+      case GTK_RESPONSE_CANCEL:
+      case GTK_RESPONSE_DELETE_EVENT:
+          gtk_widget_hide (GTK_WIDGET (dialog));
+          break;
+      default:
+          break;
+    }
+}
+
 static void
 hd_home_window_set_background_activate (HDHomeWindow *window)
 {
+  HDHomeWindowPrivate      *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget                *dialog;
+
+  dialog = g_object_new (HD_TYPE_HOME_BACKGROUND_DIALOG,
+                         "osso-context", priv->osso_context,
+                         "background-dir", HD_DESKTOP_BACKGROUNDS_PATH,
+                         "background", priv->background,
+                         NULL);
+
+  g_signal_connect_swapped (dialog, "response",
+                            G_CALLBACK (hd_home_window_set_background_reponse),
+                            window);
+
+  gtk_widget_show (dialog);
+
 }
