@@ -32,7 +32,10 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef HAVE_LIBOSSOHELP
+#include <libosso.h>
 #include <osso-helplib.h>
+#endif
 
 #include "hd-select-plugins-dialog.h"
 #include "hd-config.h"
@@ -41,6 +44,7 @@
 #define HD_SELECT_PLUGINS_DIALOG_TITLE  _("home_ti_select_applets")
 #define HD_SELECT_PLUGINS_DIALOG_OK     _("home_bd_select_applets_ok")
 #define HD_SELECT_PLUGINS_DIALOG_CANCEL _("home_bd_select_applets_cancel")
+#define HD_SELECT_PLUGINS_HELP_TOPIC    "uiframework_home_select_applets"
 
 enum
 {
@@ -219,9 +223,12 @@ hd_select_plugins_dialog_get_store (GList *loaded_plugins,
 }
 
 gint
-hd_select_plugins_dialog_run (GList *loaded_plugins,
-                              const gchar *plugin_dir,
-                              GList **selected_plugins)
+hd_select_plugins_dialog_run (GList           *loaded_plugins,
+#ifdef HAVE_LIBOSSOHELP
+                              osso_context_t  *osso_context,
+#endif
+                              const gchar     *plugin_dir,
+                              GList          **selected_plugins)
 {
   GtkWidget *dialog;
   GtkWidget *scrollwindow;
@@ -251,11 +258,11 @@ hd_select_plugins_dialog_run (GList *loaded_plugins,
 
   gtk_widget_set_size_request (dialog, HD_SELECT_PLUGINS_DIALOG_WIDTH, -1);
 
-#if 0
+#ifdef HAVE_LIBOSSOHELP
   /* Add help button to the dialog */
-  ossohelp_dialog_help_enable (GTK_DIALOG(dialog),
-                               HH_HELP_SELECT_APPLETS,
-                               osso_home);
+  ossohelp_dialog_help_enable (GTK_DIALOG (dialog),
+                               HD_SELECT_PLUGINS_HELP_TOPIC,
+                               osso_context);
 #endif
 
   scrollwindow = gtk_scrolled_window_new (NULL, NULL);
