@@ -52,8 +52,6 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmisc.h>
 
-#include <libosso.h>
-
 #include <hildon-widgets/gtk-infoprint.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
@@ -1106,16 +1104,19 @@ get_pixbuf_for_entry_info (HDEntryInfo *info)
   GError *error;
 
   error = NULL;
+  
   retval = hd_entry_info_get_app_icon (info, AS_ICON_SIZE, &error);
+
   if (!error)
     return retval;
 
-  osso_log (LOG_ERR, "Could not load icon '%s': %s\n",
-            hd_entry_info_get_app_icon_name (info),
-            error->message);
+  g_debug ("Could not load icon '%s': %s\n",
+           hd_entry_info_get_app_icon_name (info),
+           error->message);
 
   /* Fallback to default icon */
   g_clear_error (&error);
+
   retval = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 		  		     AS_MENU_DEFAULT_APP_ICON,
 				     AS_ICON_SIZE,
@@ -1124,9 +1125,9 @@ get_pixbuf_for_entry_info (HDEntryInfo *info)
   if (!error)
     return retval;
 
-  osso_log (LOG_ERR, "Could not load icon '%s': %s\n",
-            AS_MENU_DEFAULT_APP_ICON,
-            error->message);
+  g_debug ("Could not load icon '%s': %s\n",
+           AS_MENU_DEFAULT_APP_ICON,
+           error->message);
 
   g_error_free (error);
 
