@@ -42,9 +42,9 @@
 #include <hildon-base-lib/hildon-base-dnotify.h>
 #include <hildon-widgets/hildon-finger.h>
 #include <libhildondesktop/libhildonmenu.h>
+#include <libhildondesktop/hildon-desktop-panel-window.h>
 #include <libhildondesktop/hildon-thumb-menu-item.h>
 #include <libhildonwm/hd-wm.h>
-#include <libosso.h>
 
 #include "hn-others-button.h"
 #include "hn-app-switcher.h"
@@ -744,25 +744,59 @@ hn_others_button_get_menu_position (GtkMenu *menu,
   int menu_height = 0;
   int main_height = 0;
   GtkAllocation workarea = { 0, 0, 0, 0 };
+  GtkWidget *top_level;
+  HildonDesktopPanelWindowOrientation orientation = 
+      HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT;
     
   hn_others_button_get_workarea (&workarea);
+  
+  *push_in = FALSE;
+
+  top_level = gtk_widget_get_toplevel (button);
+
+  if (HILDON_DESKTOP_IS_PANEL_WINDOW (top_level))
+  {
+    g_object_get (top_level, "orientation", &orientation, NULL);
+  }
     
   gtk_widget_size_request (GTK_WIDGET (menu), &req);
 
   menu_height = req.height;
   main_height = gdk_screen_get_height (screen);
 
-  *push_in = FALSE;
-  *x =  workarea.x;
+  switch (orientation)
+  {
+    case HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT:
+        *x =  workarea.x;
 
-  if (main_height - button->allocation.y < menu_height)
-  {
-    *y = MAX(0, ((main_height - menu_height) / 2));
-  }
-  else
-  {
-    *y = button->allocation.y;
+        if (main_height - button->allocation.y < menu_height)
+          *y = MAX(0, ((main_height - menu_height) / 2));
+        else
+          *y = button->allocation.y;
+        
+        break;
+
+    case HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_RIGHT:
+        *x =  workarea.x + workarea.width - req.width;
+
+        if (main_height - button->allocation.y < menu_height)
+          *y = MAX(0, ((main_height - menu_height) / 2));
+        else
+          *y = button->allocation.y;
+        
+        break;
+    case HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_TOP:
+        *x = button->allocation.x;
+        *y = workarea.y;
+        break;
+    case HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_BOTTOM:
+        *x = button->allocation.x;
+        *y = workarea.y + workarea.height - req.height;
+        break;
+    default:
+        g_assert_not_reached ();
   }
+
 }
 
 static void
@@ -853,7 +887,7 @@ hn_others_button_press_collapser (HNOthersButton *button)
     return FALSE;
   }
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button->priv->button), TRUE);
   
   if (!TASKNAVIGATOR_ITEM (button)->menu)
     hn_others_button_create_menu (button);
